
from __future__ import absolute_import
#typing

import numpy
import torch
#overrides

from allennlp.data.fields.field import Field


class ArrayField(Field):
    u"""
    A class representing an array, which could have arbitrary dimensions.
    A batch of these arrays are padded to the max dimension length in the batch
    for each dimension.
    """
    def __init__(self, array               , padding_value      = 0)        :
        self.array = array
        self.padding_value = padding_value

    #overrides
    def get_padding_lengths(self)                  :
        return dict((u"dimension_" + unicode(i), shape)
                for i, shape in enumerate(self.array.shape))

    #overrides
    def as_tensor(self,
                  padding_lengths                ,
                  cuda_device      = -1)                :
        max_shape = [padding_lengths[u"dimension_{}".format(i)]
                     for i in range(len(padding_lengths))]

        return_array = numpy.ones(max_shape, u"float32") * self.padding_value

        # If the tensor has a different shape from the largest tensor, pad dimensions with zeros to
        # form the right shaped list of slices for insertion into the final tensor.
        slicing_shape = list(self.array.shape)
        if len(self.array.shape) < len(max_shape):
            slicing_shape = slicing_shape + [0 for _ in range(len(max_shape) - len(self.array.shape))]
        slices = tuple([slice(0, x) for x in slicing_shape])
        return_array[slices] = self.array
        tensor = torch.from_numpy(return_array)
        return tensor if cuda_device == -1 else tensor.cuda(cuda_device)

    #overrides
    def empty_field(self):  # pylint: disable=no-self-use
        # Pass the padding_value, so that any outer field, e.g., `ListField[ArrayField]` uses the
        # same padding_value in the padded ArrayFields
        return ArrayField(numpy.array([], dtype=u"float32"), padding_value=self.padding_value)


    def __str__(self)       :
        return "ArrayField with shape: {self.array.shape}."



""" LRU caching class and decorator """
from abc import abstractmethod
from abc import ABCMeta

import threading
import time
import uuid
import inspect

try:
    from itertools import izip
except ImportError:
    izip = zip


_MARKER = object()
# By default, expire items after 2**60 seconds. This fits into 64 bit
# integers and is close enough to "never" for practical purposes.
_DEFAULT_TIMEOUT = 2 ** 60


class Cache(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def clear(self):
        """Remove all entries from the cache"""

    @abstractmethod
    def get(self, key, default=None):
        """Return value for key. If not in cache, return default"""

    @abstractmethod
    def put(self, key, val):
        """Add key to the cache with value val"""

    @abstractmethod
    def invalidate(self, key):
        """Remove key from the cache"""


class UnboundedCache(Cache):
    """
    a simple unbounded cache backed by a dictionary
    """

    def __init__(self):
        self._data = dict()

    def get(self, key, default=None):
        return self._data.get(key, default)

    def clear(self):
        self._data.clear()

    def invalidate(self, key):
        try:
            del self._data[key]
        except KeyError:
            pass

    def put(self, key, val):
        self._data[key] = val


class LRUCache(Cache):
    """ Implements a pseudo-LRU algorithm (CLOCK)

    The Clock algorithm is not kept strictly to improve performance, e.g. to
    allow get() and invalidate() to work without acquiring the lock.
    """
    def __init__(self, size):
        size = int(size)
        if size < 1:
            raise ValueError('size must be >0')
        self.size = size
        self.lock = threading.Lock()
        self.hand = 0
        self.maxpos = size - 1
        self.clock_keys = None
        self.clock_refs = None
        self._data = None
        self.evictions = 0
        self.hits = 0
        self.misses = 0
        self.lookups = 0
        self.clear()

    def clear(self):
        """Remove all entries from the cache"""
        with self.lock:
            # If really clear()ing a full cache, clean up self._data first to
            # give garbage collection a chance to reduce memorey usage.
            # Instantiating "[_MARKER] * size" will temporarily have 2 lists
            # in memory -> high peak memory usage for tiny amount of time.
            # With self._data already clear, that peak should not exceed what
            # we normally use.
            self._data = {}
            size = self.size
            self.clock_keys = [_MARKER] * size
            self.clock_refs = [False] * size
            self.hand = 0
            self.evictions = 0
            self.hits = 0
            self.misses = 0
            self.lookups = 0

    def get(self, key, default=None):
        """Return value for key. If not in cache, return default"""
        self.lookups += 1
        try:
            pos, val = self._data[key]
            self.hits += 1
        except KeyError:
            self.misses += 1
            return default
        self.clock_refs[pos] = True
        return val

    def put(self, key, val):
        """Add key to the cache with value val"""
        # These do not change or they are just references, no need for locking.
        maxpos = self.maxpos
        clock_refs = self.clock_refs
        clock_keys = self.clock_keys
        data = self._data

        with self.lock:
            entry = data.get(key)
            if entry is not None:
                # We already have key. Only make sure data is up to date and
                # to remember that it was used.
                pos, old_val = entry
                if old_val is not val:
                    data[key] = (pos, val)
                self.clock_refs[pos] = True
                return
            # else: key is not yet in cache. Search place to insert it.

            hand = self.hand
            count = 0
            max_count = 107
            while 1:
                ref = clock_refs[hand]
                if ref == True:
                    clock_refs[hand] = False
                    hand += 1
                    if hand > maxpos:
                        hand = 0

                    count += 1
                    if count >= max_count:
                        # We have been searching long enough. Force eviction of
                        # next entry, no matter what its status is.
                        clock_refs[hand] = False
                else:
                    oldkey = clock_keys[hand]
                    # Maybe oldkey was not in self._data to begin with. If it
                    # was, self.invalidate() in another thread might have
                    # already removed it. del() would raise KeyError, so pop().
                    oldentry = data.pop(oldkey, _MARKER)
                    if oldentry is not _MARKER:
                        self.evictions += 1
                    clock_keys[hand] = key
                    clock_refs[hand] = True
                    data[key] = (hand, val)
                    hand += 1
                    if hand > maxpos:
                        hand = 0
                    self.hand = hand
                    break

    def invalidate(self, key):
        """Remove key from the cache"""
        # pop with default arg will not raise KeyError
        entry = self._data.pop(key, _MARKER)
        if entry is not _MARKER:
            # We have no lock, but worst thing that can happen is that we
            # set another key's entry to False.
            self.clock_refs[entry[0]] = False
        # else: key was not in cache. Nothing to do.


class ExpiringLRUCache(Cache):
    """ Implements a pseudo-LRU algorithm (CLOCK) with expiration times

    The Clock algorithm is not kept strictly to improve performance, e.g. to
    allow get() and invalidate() to work without acquiring the lock.
    """
    def __init__(self, size, default_timeout=_DEFAULT_TIMEOUT):
        self.default_timeout = default_timeout
        size = int(size)
        if size < 1:
            raise ValueError('size must be >0')
        self.size = size
        self.lock = threading.Lock()
        self.hand = 0
        self.maxpos = size - 1
        self.clock_keys = None
        self.clock_refs = None
        self._data = None
        self.evictions = 0
        self.hits = 0
        self.misses = 0
        self.lookups = 0
        self.clear()

    def clear(self):
        """Remove all entries from the cache"""
        with self.lock:
            # If really clear()ing a full cache, clean up self._data first to
            # give garbage collection a chance to reduce memorey usage.
            # Instantiating "[_MARKER] * size" will temporarily have 2 lists
            # in memory -> high peak memory usage for tiny amount of time.
            # With self._data already clear, that peak should not exceed what
            # we normally use.
            # self._data contains (pos, val, expires) triplets
            self._data = {}
            size = self.size
            self.clock_keys = [_MARKER] * size
            self.clock_refs = [False] * size
            self.hand = 0
            self.evictions = 0
            self.hits = 0
            self.misses = 0
            self.lookups = 0

    def get(self, key, default=None):
        """Return value for key. If not in cache or expired, return default"""
        self.lookups += 1
        try:
            pos, val, expires = self._data[key]
        except KeyError:
            self.misses += 1
            return default
        if expires > time.time():
            # cache entry still valid
            self.hits += 1
            self.clock_refs[pos] = True
            return val
        else:
            # cache entry has expired. Make sure the space in the cache can
            # be recycled soon.
            self.misses += 1
            self.clock_refs[pos] = False
            return default

    def put(self, key, val, timeout=None):
        """Add key to the cache with value val

        key will expire in $timeout seconds. If key is already in cache, val
        and timeout will be updated.
        """
        # These do not change or they are just references, no need for locking.
        maxpos = self.maxpos
        clock_refs = self.clock_refs
        clock_keys = self.clock_keys
        data = self._data
        lock = self.lock
        if timeout is None:
            timeout = self.default_timeout

        with self.lock:
            entry = data.get(key)
            if entry is not None:
                # We already have key. Only make sure data is up to date and
                # to remember that it was used.
                pos = entry[0]
                data[key] = (pos, val, time.time() + timeout)
                clock_refs[pos] = True
                return
            # else: key is not yet in cache. Search place to insert it.

            hand = self.hand
            count = 0
            max_count = 107
            while 1:
                ref = clock_refs[hand]
                if ref == True:
                    clock_refs[hand] = False
                    hand += 1
                    if hand > maxpos:
                        hand = 0

                    count += 1
                    if count >= max_count:
                        # We have been searching long enough. Force eviction of
                        # next entry, no matter what its status is.
                        clock_refs[hand] = False
                else:
                    oldkey = clock_keys[hand]
                    # Maybe oldkey was not in self._data to begin with. If it
                    # was, self.invalidate() in another thread might have
                    # already removed it. del() would raise KeyError, so pop().
                    oldentry = data.pop(oldkey, _MARKER)
                    if oldentry is not _MARKER:
                        self.evictions += 1
                    clock_keys[hand] = key
                    clock_refs[hand] = True
                    data[key] = (hand, val, time.time() + timeout)
                    hand += 1
                    if hand > maxpos:
                        hand = 0
                    self.hand = hand
                    break

    def invalidate(self, key):
        """Remove key from the cache"""
        # pop with default arg will not raise KeyError
        entry = self._data.pop(key, _MARKER)
        if entry is not _MARKER:
            # We have no lock, but worst thing that can happen is that we
            # set another key's entry to False.
            self.clock_refs[entry[0]] = False
        # else: key was not in cache. Nothing to do.


class lru_cache(object):
    """ Decorator for LRU-cached function

    timeout parameter specifies after how many seconds a cached entry should
    be considered invalid.
    """
    def __init__(self,
                 maxsize,
                 cache=None, # cache is an arg to serve tests
                 timeout=None,
                 ignore_unhashable_args=False,
                 real_func=None,
                 remove_self=False):
        if cache is None:
            if maxsize is None:
                cache = UnboundedCache()
            elif timeout is None:
                cache = LRUCache(maxsize)
            else:
                cache = ExpiringLRUCache(maxsize, default_timeout=timeout)
        self.real_func = real_func
        self.remove_self = remove_self
        self.cache = cache
        self._ignore_unhashable_args = ignore_unhashable_args

    def __call__(self, func):
        cache = self.cache
        marker = _MARKER

        def cached_wrapper(*args, **kwargs):
            def list_to_tuple(list_of_lists):
                return tuple(tuple(x) if isinstance(x, list) else x 
                    for x in list_of_lists)

            def get_default_args(remove_self, func, rargs, kwargs):
                """
                returns a dictionary of arg_name:default_values for the input function
                """
                if hasattr(inspect, 'getfullargspec'):
                    argspec = inspect.getfullargspec(func)
                    args = argspec.args
                    defaults = argspec.defaults
                else:
                    args, _, _, defaults = inspect.getargspec(func)
                args = args or []
                defaults = defaults or []
                if remove_self:
                    args = args[1:]

                if defaults is not None:
                    ddict = dict(zip(args[-len(defaults):], defaults))
                    new_kwargs = dict(zip(args[-len(defaults):], rargs[len(args)-len(defaults):]))
                    ddict.update(new_kwargs)
                    ddict.update(kwargs)
                else:
                    ddict = dict()
                new_rargs = rargs[:len(args)-len(defaults)]
                return new_rargs, ddict 

            try:
                args, kwargs = get_default_args(self.remove_self, self.real_func or func, args, kwargs)
                args_t = tuple(list_to_tuple(arg) if isinstance(arg, list) else arg for arg in args)
                kwargs_t = frozenset(((kwarg[0], list_to_tuple(kwarg[1])) if isinstance(kwarg[1], list) else kwarg for kwarg in kwargs.items()))
                key = (args_t, kwargs_t) if kwargs else args_t
                val = cache.get(key, marker)
            except TypeError as e:
                if self._ignore_unhashable_args:
                    return func(*args, **kwargs)
                else:
                    raise e
            else:
                if val is marker:
                    val = func(*args, **kwargs)
                    cache.put(key, val)
                return val

        def _maybe_copy(source, target, attr):
            value = getattr(source, attr, source)
            if value is not source:
                setattr(target, attr, value)

        _maybe_copy(func, cached_wrapper, '__module__')
        _maybe_copy(func, cached_wrapper, '__name__')
        _maybe_copy(func, cached_wrapper, '__doc__')
        cached_wrapper._cache = cache
        return cached_wrapper


class CacheMaker(object):
    """Generates decorators that can be cleared later
    """
    def __init__(self, maxsize=None, timeout=_DEFAULT_TIMEOUT):
        """Create cache decorator factory.

        - maxsize : the default size for created caches.

        - timeout : the defaut expiraiton time for created caches.
        """
        self._maxsize = maxsize
        self._timeout = timeout
        self._cache = {}

    def _resolve_setting(self, name=None, maxsize=None, timeout=None):
        if name is None:
            while True:
                name = str(uuid.uuid4())
                ## the probability of collision is so low ....
                if name not in self._cache:  # pragma: NO COVER
                    break

        if name in self._cache:
            raise KeyError("cache %s already in use" % name)

        if maxsize is None:
            maxsize = self._maxsize

        if maxsize is None:
            raise ValueError("Cache must have a maxsize set")

        if timeout is None:
            timeout = self._timeout

        return name, maxsize, timeout

    def memoized(self, name=None):
        name, maxsize, _ = self._resolve_setting(name, 0)
        cache = self._cache[name] = UnboundedCache()
        return lru_cache(None, cache)

    def lrucache(self, name=None, maxsize=None):
        """Named arguments:
        
        - name (optional) is a string, and should be unique amongst all caches

        - maxsize (optional) is an int, overriding any default value set by
          the constructor
        """
        name, maxsize, _ = self._resolve_setting(name, maxsize)
        cache = self._cache[name] = LRUCache(maxsize)
        return lru_cache(maxsize, cache)

    def expiring_lrucache(self, name=None, maxsize=None, timeout=None):
        """Named arguments:

        - name (optional) is a string, and should be unique amongst all caches

        - maxsize (optional) is an int, overriding any default value set by
          the constructor

        - timeout (optional) is an int, overriding any default value set by
          the constructor or the default value (%d seconds)
        """ % _DEFAULT_TIMEOUT
        name, maxsize, timeout = self._resolve_setting(name, maxsize, timeout)
        cache = self._cache[name] = ExpiringLRUCache(maxsize, timeout)
        return lru_cache(maxsize, cache, timeout)

    def clear(self, *names):
        """Clear the given cache(s).
        
        If no 'names' are passed, clear all caches.
        """
        if len(names) == 0:
            names = self._cache.keys()

        for name in names:
            self._cache[name].clear()





from __future__ import division
from __future__ import absolute_import
#typing

#overrides
import torch

from allennlp.training.metrics.metric import Metric


class Entropy(Metric):

    def __init__(self)        :
        self._entropy = 0.0
        self._count = 0

    #overrides
    def __call__(self,  # type: ignore
                 logits              ,
                 mask                         = None):
        u"""
        Parameters
        ----------
        logits : ``torch.Tensor``, required.
            A tensor of unnormalized log probabilities of shape (batch_size, ..., num_classes).
        mask: ``torch.Tensor``, optional (default = None).
            A masking tensor of shape (batch_size, ...).
        """
        logits, mask = self.unwrap_to_tensors(logits, mask)

        if mask is None:
            mask = torch.ones(logits.size()[:-1])

        log_probs = torch.nn.functional.log_softmax(logits, dim=-1)
        probabilities = torch.exp(log_probs) * mask.unsqueeze(-1)
        weighted_negative_likelihood = - log_probs * probabilities
        entropy = weighted_negative_likelihood.sum(-1)

        self._entropy += entropy.sum() / mask.sum()
        self._count += 1

    #overrides
    def get_metric(self, reset       = False):
        u"""
        Returns
        -------
        The scalar average entropy.
        """
        average_value = self._entropy / self._count if self._count > 0 else 0
        if reset:
            self.reset()
        return average_value

    #overrides
    def reset(self):
        self._entropy = 0.0
        self._count = 0

Entropy = Metric.register(u"entropy")(Entropy)



u"""
A :class:`~allennlp.predictors.predictor.Predictor` is
a wrapper for an AllenNLP ``Model``
that makes JSON predictions using JSON inputs. If you
want to serve up a model through the web service
(or using ``allennlp.commands.predict``), you'll need
a ``Predictor`` that wraps it.
"""

from __future__ import absolute_import
from allennlp.predictors.predictor import Predictor
from allennlp.predictors.bidaf import BidafPredictor
from allennlp.predictors.constituency_parser import ConstituencyParserPredictor
from allennlp.predictors.biaffine_dependency_parser import BiaffineDependencyParserPredictor
from allennlp.predictors.coref import CorefPredictor
from allennlp.predictors.decomposable_attention import DecomposableAttentionPredictor
from allennlp.predictors.semantic_role_labeler import SemanticRoleLabelerPredictor
from allennlp.predictors.sentence_tagger import SentenceTaggerPredictor
from allennlp.predictors.simple_seq2seq import SimpleSeq2SeqPredictor
from allennlp.predictors.wikitables_parser import WikiTablesParserPredictor
from allennlp.predictors.nlvr_parser import NlvrParserPredictor





from __future__ import division
from __future__ import absolute_import
import logging
#typing
from collections import defaultdict
import itertools
import math
import random

import torch

from allennlp.common.registrable import Registrable
from allennlp.common.util import is_lazy, lazy_groups_of, ensure_list
from allennlp.data.dataset import Batch
from allennlp.data.fields import MetadataField
from allennlp.data.instance import Instance
from allennlp.data.vocabulary import Vocabulary

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


def add_epoch_number(batch       , epoch     )         :
    u"""
    Add the epoch number to the batch instances as a MetadataField.
    """
    for instance in batch.instances:
        instance.fields[u'epoch_num'] = MetadataField(epoch)
    return batch


class DataIterator(Registrable):
    u"""
    An abstract ``DataIterator`` class. ``DataIterators`` must override ``_create_batches()``.

    Parameters
    ----------
    batch_size : ``int``, optional, (default = 32)
        The size of each batch of instances yielded when calling the iterator.
    instances_per_epoch : ``int``, optional, (default = None)
        If specified, each epoch will consist of precisely this many instances.
        If not specified, each epoch will consist of a single pass through the dataset.
    max_instances_in_memory : ``int``, optional, (default = None)
        If specified, the iterator will load this many instances at a time into an
        in-memory list and then produce batches from one such list at a time. This
        could be useful if your instances are read lazily from disk.
    cache_instances : ``bool``, optional, (default = False)
        If true, the iterator will cache the tensorized instances in memory.
        If false, it will do the tensorization anew each iteration.
    track_epoch : ``bool``, optional, (default = False)
        If true, each instance will get a ``MetadataField`` containing the epoch number.
    maximum_samples_per_batch : ``Tuple[str, int]``, (default = None)
        If specified, then is a tuple (padding_key, limit) and we will
        shrink the batch size for very long sequences such that
        batch_size * sequence_length <= limit where sequence_length is given
        by the padding_key.
    """
    default_implementation = u'bucket'

    def __init__(self,
                 batch_size      = 32,
                 instances_per_epoch      = None,
                 max_instances_in_memory      = None,
                 cache_instances       = False,
                 track_epoch       = False,
                 maximum_samples_per_batch                  = None)        :
        self.vocab = None

        self._batch_size = batch_size
        self._max_instances_in_memory = max_instances_in_memory
        self._instances_per_epoch = instances_per_epoch
        self._maximum_samples_per_batch = maximum_samples_per_batch

        # We might want to cache the instances in memory.
        self._cache_instances = cache_instances
        self._cache = defaultdict(list)

        # We also might want to add the epoch number to each instance.
        self._track_epoch = track_epoch
        self._epochs = defaultdict(int)

        # We also might want to keep track of cursors;
        # for example, if each epoch represents less than one pass through the dataset,
        # we want to remember where we left off. As `Iterator`s are not necessarily hashable,
        # we use their id() as the key.
        self._cursors = {}


    def __call__(self,
                 instances                    ,
                 num_epochs      = None,
                 shuffle       = True,
                 cuda_device      = -1)                        :
        u"""
        Returns a generator that yields batches over the given dataset
        for the given number of epochs. If ``num_epochs`` is not specified,
        it will yield batches forever.

        Parameters
        ----------
        instances : ``Iterable[Instance]``
            The instances in the dataset. IMPORTANT: this must be able to be
            iterated over *multiple times*. That is, it must be either a List
            or some other object whose ``__iter__`` method returns a fresh iterator
            each time it's called.
        num_epochs : ``int``, optional (default=``None``)
            How times should we iterate over this dataset?  If ``None``, we will iterate over it
            forever.
        shuffle : ``bool``, optional (default=``True``)
            If ``True``, we will shuffle the instances in ``dataset`` before constructing batches
            and iterating over the data.
        cuda_device : ``int``
            If cuda_device >= 0, GPUs are available and Pytorch was compiled with CUDA support, the
            tensor will be copied to the cuda_device specified.
        """
        # Instances is likely to be a list, which cannot be used as a key,
        # so we take the object id instead.
        key = id(instances)
        starting_epoch = self._epochs[key]

        if num_epochs is None:
            epochs                = itertools.count(starting_epoch)
        else:
            epochs = range(starting_epoch, starting_epoch + num_epochs)

        for epoch in epochs:
            self._epochs[key] = epoch

            if self._cache_instances and key in self._cache:
                # Serve the results from the cache.
                tensor_dicts = self._cache[key]

                if shuffle:
                    random.shuffle(tensor_dicts)
                for tensor_dict in tensor_dicts:
                    if self._track_epoch:
                        # The tensor_dict already has an "epoch_num" tensor,
                        # so just fill it with the right value.
                        epoch_tensor               = tensor_dict[u'epoch_num']
                        epoch_tensor.fill_(epoch)
                    yield tensor_dict
            else:
                batches = self._create_batches(instances, shuffle)

                # Should we add the instances to the cache this epoch?
                add_to_cache = self._cache_instances and key not in self._cache

                for batch in batches:
                    if self._track_epoch:
                        add_epoch_number(batch, epoch)

                    if self.vocab is not None:
                        batch.index_instances(self.vocab)

                    padding_lengths = batch.get_padding_lengths()
                    logger.debug(u"Batch padding lengths: %s", unicode(padding_lengths))
                    logger.debug(u"Batch size: %d", len(batch.instances))
                    tensor_dict = batch.as_tensor_dict(padding_lengths,
                                                       cuda_device=cuda_device)

                    if add_to_cache:
                        self._cache[key].append(tensor_dict)

                    yield tensor_dict

    def _take_instances(self,
                        instances                    ,
                        max_instances                = None)                      :
        u"""
        Take the next `max_instances` instances from the given dataset.
        If `max_instances` is `None`, then just take all instances from the dataset.
        If `max_instances` is not `None`, each call resumes where the previous one
        left off, and when you get to the end of the dataset you start again from the beginning.
        """
        # If max_instances isn't specified, just iterate once over the whole dataset
        if max_instances is None:
            _i = iter(instances)
            while True:
                yield _i.next()
        else:
            # If we don't have a cursor for this dataset, create one. We use ``id()``
            # for the key because ``instances`` could be a list, which can't be used as a key.
            key = id(instances)
            iterator = self._cursors.get(key, iter(instances))

            while max_instances > 0:
                try:
                    # If there are instances left on this iterator,
                    # yield one and decrement max_instances.
                    yield next(iterator)
                    max_instances -= 1
                except StopIteration:
                    # None left, so start over again at the beginning of the dataset.
                    iterator = iter(instances)

            # We may have a new iterator, so update the cursor.
            self._cursors[key] = iterator

    def _memory_sized_lists(self,
                            instances                    )                            :
        u"""
        Breaks the dataset into "memory-sized" lists of instances,
        which it yields up one at a time until it gets through a full epoch.

        For example, if the dataset is already an in-memory list, and each epoch
        represents one pass through the dataset, it just yields back the dataset.
        Whereas if the dataset is lazily read from disk and we've specified to
        load 1000 instances at a time, then it yields lists of 1000 instances each.
        """
        lazy = is_lazy(instances)

        # Get an iterator over the next epoch worth of instances.
        iterator = self._take_instances(instances, self._instances_per_epoch)

        # We have four different cases to deal with:

        # With lazy instances and no guidance about how many to load into memory,
        # we just load ``batch_size`` instances at a time:
        if lazy and self._max_instances_in_memory is None:
            _i = lazy_groups_of(iterator, self._batch_size)
            while True:
                yield _i.next()
        # If we specified max instances in memory, lazy or not, we just
        # load ``max_instances_in_memory`` instances at a time:
        elif self._max_instances_in_memory is not None:
            _i = lazy_groups_of(iterator, self._max_instances_in_memory)
            while True:
                yield _i.next()
        # If we have non-lazy instances, and we want all instances each epoch,
        # then we just yield back the list of instances:
        elif self._instances_per_epoch is None:
            yield ensure_list(instances)
        # In the final case we have non-lazy instances, we want a specific number
        # of instances each epoch, and we didn't specify how to many instances to load
        # into memory. So we convert the whole iterator to a list:
        else:
            yield list(iterator)


    def _ensure_batch_is_sufficiently_small(self, batch_instances                    )                        :
        u"""
        If self._maximum_samples_per_batch is specified, then split the batch into smaller
        sub-batches if it exceeds the maximum size.
        """
        if self._maximum_samples_per_batch is None:
            return [list(batch_instances)]

        # check if we need to break into smaller chunks
        key, limit = self._maximum_samples_per_batch
        padding_length = -1
        list_batch_instances = list(batch_instances)
        for instance in list_batch_instances:
            field_lengths = instance.get_padding_lengths()
            for _, lengths in list(field_lengths.items()):
                try:
                    padding_length = max(padding_length,
                                         lengths[key])
                except KeyError:
                    pass

        if padding_length * len(list_batch_instances) > limit:
            # need to shrink
            num_samples = padding_length * len(list_batch_instances)
            num_shrunk_batches = math.ceil(num_samples / float(limit))
            shrunk_batch_size = math.ceil(len(list_batch_instances) / num_shrunk_batches)
            shrunk_batches = []
            start = 0
            while start < len(list_batch_instances):
                end = start + shrunk_batch_size
                shrunk_batches.append(list_batch_instances[start:end])
                start = end
            return shrunk_batches
        else:
            return [list_batch_instances]


    def get_num_batches(self, instances                    )       :
        u"""
        Returns the number of batches that ``dataset`` will be split into; if you want to track
        progress through the batch with the generator produced by ``__call__``, this could be
        useful.
        """
        if is_lazy(instances) and self._instances_per_epoch is None:
            # Unable to compute num batches, so just return 1.
            return 1
        elif self._instances_per_epoch is not None:
            return math.ceil(self._instances_per_epoch / self._batch_size)
        else:
            # Not lazy, so can compute the list length.
            return math.ceil(len(ensure_list(instances)) / self._batch_size)


    def _create_batches(self, instances                    , shuffle      )                   :
        u"""
        This method should return one epoch worth of batches.
        """
        raise NotImplementedError

    def index_with(self, vocab            ):
        self.vocab = vocab




from __future__ import absolute_import
#typing

import numpy
#overrides
import torch
from torch.nn.modules.linear import Linear
import torch.nn.functional as F

from allennlp.common.checks import check_dimensions_match
from allennlp.data import Vocabulary
from allennlp.modules import Seq2SeqEncoder, TimeDistributed, TextFieldEmbedder
from allennlp.models.model import Model
from allennlp.nn import InitializerApplicator, RegularizerApplicator
from allennlp.nn.util import get_text_field_mask, sequence_cross_entropy_with_logits
from allennlp.training.metrics import CategoricalAccuracy


class SimpleTagger(Model):
    u"""
    This ``SimpleTagger`` simply encodes a sequence of text with a stacked ``Seq2SeqEncoder``, then
    predicts a tag for each token in the sequence.

    Parameters
    ----------
    vocab : ``Vocabulary``, required
        A Vocabulary, required in order to compute sizes for input/output projections.
    text_field_embedder : ``TextFieldEmbedder``, required
        Used to embed the ``tokens`` ``TextField`` we get as input to the model.
    encoder : ``Seq2SeqEncoder``
        The encoder (with its own internal stacking) that we will use in between embedding tokens
        and predicting output tags.
    initializer : ``InitializerApplicator``, optional (default=``InitializerApplicator()``)
        Used to initialize the model parameters.
    regularizer : ``RegularizerApplicator``, optional (default=``None``)
        If provided, will be used to calculate the regularization penalty during training.
    """

    def __init__(self, vocab            ,
                 text_field_embedder                   ,
                 encoder                ,
                 initializer                        = InitializerApplicator(),
                 regularizer                                  = None)        :
        super(SimpleTagger, self).__init__(vocab, regularizer)

        self.text_field_embedder = text_field_embedder
        self.num_classes = self.vocab.get_vocab_size(u"labels")
        self.encoder = encoder
        self.tag_projection_layer = TimeDistributed(Linear(self.encoder.get_output_dim(),
                                                           self.num_classes))

        check_dimensions_match(text_field_embedder.get_output_dim(), encoder.get_input_dim(),
                               u"text field embedding dim", u"encoder input dim")
        self.metrics = {
                u"accuracy": CategoricalAccuracy(),
                u"accuracy3": CategoricalAccuracy(top_k=3)
        }

        initializer(self)

    #overrides
    def forward(self,  # type: ignore
                tokens                             ,
                tags                   = None,
                metadata                       = None)                           :
        # pylint: disable=arguments-differ
        u"""
        Parameters
        ----------
        tokens : Dict[str, torch.LongTensor], required
            The output of ``TextField.as_array()``, which should typically be passed directly to a
            ``TextFieldEmbedder``. This output is a dictionary mapping keys to ``TokenIndexer``
            tensors.  At its most basic, using a ``SingleIdTokenIndexer`` this is: ``{"tokens":
            Tensor(batch_size, num_tokens)}``. This dictionary will have the same keys as were used
            for the ``TokenIndexers`` when you created the ``TextField`` representing your
            sequence.  The dictionary is designed to be passed directly to a ``TextFieldEmbedder``,
            which knows how to combine different word representations into a single vector per
            token in your input.
        tags : torch.LongTensor, optional (default = None)
            A torch tensor representing the sequence of integer gold class labels of shape
            ``(batch_size, num_tokens)``.
        metadata : ``List[Dict[str, Any]]``, optional, (default = None)
            metadata containg the original words in the sentence to be tagged under a 'words' key.

        Returns
        -------
        An output dictionary consisting of:
        logits : torch.FloatTensor
            A tensor of shape ``(batch_size, num_tokens, tag_vocab_size)`` representing
            unnormalised log probabilities of the tag classes.
        class_probabilities : torch.FloatTensor
            A tensor of shape ``(batch_size, num_tokens, tag_vocab_size)`` representing
            a distribution of the tag classes per word.
        loss : torch.FloatTensor, optional
            A scalar loss to be optimised.

        """
        embedded_text_input = self.text_field_embedder(tokens)
        batch_size, sequence_length, _ = embedded_text_input.size()
        mask = get_text_field_mask(tokens)
        encoded_text = self.encoder(embedded_text_input, mask)

        logits = self.tag_projection_layer(encoded_text)
        reshaped_log_probs = logits.view(-1, self.num_classes)
        class_probabilities = F.softmax(reshaped_log_probs, dim=-1).view([batch_size,
                                                                          sequence_length,
                                                                          self.num_classes])

        output_dict = {u"logits": logits, u"class_probabilities": class_probabilities}

        if tags is not None:
            loss = sequence_cross_entropy_with_logits(logits, tags, mask)
            for metric in list(self.metrics.values()):
                metric(logits, tags, mask.float())
            output_dict[u"loss"] = loss

        if metadata is not None:
            output_dict[u"words"] = [x[u"words"] for x in metadata]
        return output_dict

    #overrides
    def decode(self, output_dict                         )                           :
        u"""
        Does a simple position-wise argmax over each token, converts indices to string labels, and
        adds a ``"tags"`` key to the dictionary with the result.
        """
        all_predictions = output_dict[u'class_probabilities']
        all_predictions = all_predictions.cpu().data.numpy()
        if all_predictions.ndim == 3:
            predictions_list = [all_predictions[i] for i in range(all_predictions.shape[0])]
        else:
            predictions_list = [all_predictions]
        all_tags = []
        for predictions in predictions_list:
            argmax_indices = numpy.argmax(predictions, axis=-1)
            tags = [self.vocab.get_token_from_index(x, namespace=u"labels")
                    for x in argmax_indices]
            all_tags.append(tags)
        output_dict[u'tags'] = all_tags
        return output_dict

    #overrides
    def get_metrics(self, reset       = False)                    :
        return dict((metric_name, metric.get_metric(reset)) for metric_name, metric in list(self.metrics.items()))

SimpleTagger = Model.register(u"simple_tagger")(SimpleTagger)



# pylint: disable=no-self-use,invalid-name,protected-access,too-many-public-methods

from __future__ import absolute_import
from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Token
from allennlp.semparse.contexts import TableQuestionKnowledgeGraph


class TestTableQuestionKnowledgeGraph(AllenNlpTestCase):
    def test_read_from_json_handles_simple_cases(self):
        json = {
                u'question': [Token(x) for x in [u'where', u'is', u'mersin', u'?']],
                u'columns': [u'Name in English', u'Location'],
                u'cells': [[u'Paradeniz', u'Mersin'],
                          [u'Lake Gala', u'Edirne']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:cell.mersin'])
        assert graph.entities == [u'-1', u'0', u'1', u'fb:cell.edirne', u'fb:cell.lake_gala',
                                  u'fb:cell.mersin', u'fb:cell.paradeniz', u'fb:row.row.location',
                                  u'fb:row.row.name_in_english']
        assert neighbors == set([u'fb:row.row.location'])
        neighbors = set(graph.neighbors[u'fb:row.row.name_in_english'])
        assert neighbors == set([u'fb:cell.paradeniz', u'fb:cell.lake_gala'])
        assert graph.entity_text[u'fb:cell.edirne'] == u'Edirne'
        assert graph.entity_text[u'fb:cell.lake_gala'] == u'Lake Gala'
        assert graph.entity_text[u'fb:cell.mersin'] == u'Mersin'
        assert graph.entity_text[u'fb:cell.paradeniz'] == u'Paradeniz'
        assert graph.entity_text[u'fb:row.row.location'] == u'Location'
        assert graph.entity_text[u'fb:row.row.name_in_english'] == u'Name in English'

        # These are default numbers that should always be in the graph.
        assert graph.neighbors[u'-1'] == []
        assert graph.neighbors[u'0'] == []
        assert graph.neighbors[u'1'] == []
        assert graph.entity_text[u'-1'] == u'-1'
        assert graph.entity_text[u'0'] == u'0'
        assert graph.entity_text[u'1'] == u'1'

    def test_read_from_json_replaces_newlines(self):
        # The csv -> tsv conversion renders '\n' as r'\n' (with a literal slash character), that
        # gets read in a two characters instead of one.  We need to make sure we convert it back to
        # one newline character, so our splitting and other processing works correctly.
        json = {
                u'question': [Token(x) for x in [u'where', u'is', u'mersin', u'?']],
                u'columns': [u'Name\\nin English', u'Location'],
                u'cells': [[u'Paradeniz', u'Mersin'],
                          [u'Lake\\nGala', u'Edirne']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        assert graph.entities == [u'-1', u'0', u'1', u'fb:cell.edirne', u'fb:cell.lake_gala',
                                  u'fb:cell.mersin', u'fb:cell.paradeniz', u'fb:part.gala',
                                  u'fb:part.lake', u'fb:part.paradeniz', u'fb:row.row.location',
                                  u'fb:row.row.name_in_english']
        assert graph.entity_text[u'fb:row.row.name_in_english'] == u'Name\nin English'

    def test_read_from_json_splits_columns_when_necessary(self):
        json = {
                u'question': [Token(x) for x in [u'where', u'is', u'mersin', u'?']],
                u'columns': [u'Name in English', u'Location'],
                u'cells': [[u'Paradeniz', u'Mersin with spaces'],
                          [u'Lake, Gala', u'Edirne']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        assert graph.entities == [u'-1', u'0', u'1', u'fb:cell.edirne', u'fb:cell.lake_gala',
                                  u'fb:cell.mersin_with_spaces', u'fb:cell.paradeniz', u'fb:part.gala',
                                  u'fb:part.lake', u'fb:part.paradeniz', u'fb:row.row.location',
                                  u'fb:row.row.name_in_english']
        assert graph.neighbors[u'fb:part.lake'] == []
        assert graph.neighbors[u'fb:part.gala'] == []
        assert graph.neighbors[u'fb:part.paradeniz'] == []

    def test_read_from_json_handles_numbers_in_question(self):
        # The TSV file we use has newlines converted to "\n", not actual escape characters.  We
        # need to be sure we catch this.
        json = {
                u'question': [Token(x) for x in [u'one', u'4']],
                u'columns': [],
                u'cells': []
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        assert graph.neighbors[u'1'] == []
        assert graph.neighbors[u'4'] == []
        assert graph.entity_text[u'1'] == u'one'
        assert graph.entity_text[u'4'] == u'4'

    def test_get_cell_parts_returns_cell_text_on_simple_cells(self):
        assert TableQuestionKnowledgeGraph._get_cell_parts(u'Team') == [(u'fb:part.team', u'Team')]
        assert TableQuestionKnowledgeGraph._get_cell_parts(u'2006') == [(u'fb:part.2006', u'2006')]
        assert TableQuestionKnowledgeGraph._get_cell_parts(u'Wolfe Tones') == [(u'fb:part.wolfe_tones',
                                                                               u'Wolfe Tones')]

    def test_get_cell_parts_splits_on_commas(self):
        parts = TableQuestionKnowledgeGraph._get_cell_parts(u'United States, Los Angeles')
        assert set(parts) == set([(u'fb:part.united_states', u'United States'),
                              (u'fb:part.los_angeles', u'Los Angeles')])

    def test_get_cell_parts_on_past_failure_cases(self):
        parts = TableQuestionKnowledgeGraph._get_cell_parts(u'Checco D\'Angelo\n "Jimmy"')
        assert set(parts) == set([(u'fb:part.checco_d_angelo', u"Checco D\'Angelo"),
                              (u'fb:part._jimmy', u'"Jimmy"')])

    def test_get_cell_parts_handles_multiple_splits(self):
        parts = TableQuestionKnowledgeGraph._get_cell_parts(u'this, has / lots\n of , commas')
        assert set(parts) == set([(u'fb:part.this', u'this'),
                              (u'fb:part.has', u'has'),
                              (u'fb:part.lots', u'lots'),
                              (u'fb:part.of', u'of'),
                              (u'fb:part.commas', u'commas')])

    def test_should_split_column_returns_false_when_all_text_is_simple(self):
        assert TableQuestionKnowledgeGraph._should_split_column_cells([u'Team', u'2006', u'Wolfe Tones']) is False

    def test_should_split_column_returns_true_when_one_input_is_splitable(self):
        assert TableQuestionKnowledgeGraph._should_split_column_cells([u'Team, 2006', u'Wolfe Tones']) is True

    def test_read_from_json_handles_diacritics(self):
        json = {
                u'question': [],
                u'columns': [u'Name in English', u'Name in Turkish', u'Location'],
                u'cells': [[u'Lake Van', u'Van Gölü', u'Mersin'],
                          [u'Lake Gala', u'Gala Gölü', u'Edirne']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.name_in_turkish'])
        assert neighbors == set([u'fb:cell.van_golu', u'fb:cell.gala_golu'])

        json = {
                u'question': [],
                u'columns': [u'Notes'],
                u'cells': [[u'Ordained as a priest at\nReșița on March, 29th 1936']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.notes'])
        assert neighbors == set([u'fb:cell.ordained_as_a_priest_at_resita_on_march_29th_1936'])

        json = {
                u'question': [],
                u'columns': [u'Player'],
                u'cells': [[u'Mateja Kežman']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.player'])
        assert neighbors == set([u'fb:cell.mateja_kezman'])

        json = {
                u'question': [],
                u'columns': [u'Venue'],
                u'cells': [[u'Arena Națională, Bucharest, Romania']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.venue'])
        assert neighbors == set([u'fb:cell.arena_nationala_bucharest_romania'])

    def test_read_from_json_handles_newlines_in_columns(self):
        # The TSV file we use has newlines converted to "\n", not actual escape characters.  We
        # need to be sure we catch this.
        json = {
                u'question': [],
                u'columns': [u'Peak\\nAUS', u'Peak\\nNZ'],
                u'cells': [[u'1', u'2'],
                          [u'3', u'4']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.peak_aus'])
        assert neighbors == set([u'fb:cell.1', u'fb:cell.3'])
        neighbors = set(graph.neighbors[u'fb:row.row.peak_nz'])
        assert neighbors == set([u'fb:cell.2', u'fb:cell.4'])
        neighbors = set(graph.neighbors[u'fb:cell.1'])
        assert neighbors == set([u'fb:row.row.peak_aus'])

        json = {
                u'question': [],
                u'columns': [u'Title'],
                u'cells': [[u'Dance of the\\nSeven Veils']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.title'])
        assert neighbors == set([u'fb:cell.dance_of_the_seven_veils'])

    def test_read_from_json_handles_diacritics_and_newlines(self):
        json = {
                u'question': [],
                u'columns': [u'Notes'],
                u'cells': [[u'8 districts\nFormed from Orūzgān Province in 2004']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.notes'])
        assert neighbors == set([u'fb:cell.8_districts_formed_from_oruzgan_province_in_2004'])

    def test_read_from_json_handles_crazy_unicode(self):
        json = {
                u'question': [],
                u'columns': [u'Town'],
                u'cells': [[u'Viðareiði'],
                          [u'Funningsfjørður'],
                          [u'Froðba']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.town'])
        assert neighbors == set([
                u'fb:cell.funningsfj_r_ur',
                u'fb:cell.vi_arei_i',
                u'fb:cell.fro_ba',])

        json = {
                u'question': [],
                u'columns': [u'Fate'],
                u'cells': [[u'Sunk at 45°00′N 11°21′W﻿ / ﻿45.000°N 11.350°W'],
                          [u'66°22′32″N 29°20′19″E﻿ / ﻿66.37556°N 29.33861°E']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.fate'])
        assert neighbors == set([u'fb:cell.sunk_at_45_00_n_11_21_w_45_000_n_11_350_w',
                             u'fb:cell.66_22_32_n_29_20_19_e_66_37556_n_29_33861_e'])

        json = {
                u'question': [],
                u'columns': [u'€0.01', u'Σ Points'],
                u'cells': [[u'6,000', u'9.5']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row._0_01'])
        assert neighbors == set([u'fb:cell.6_000'])
        neighbors = set(graph.neighbors[u'fb:row.row._points'])
        assert neighbors == set([u'fb:cell.9_5'])

        json = {
                u'question': [],
                u'columns': [u'Division'],
                u'cells': [[u'1ª Aut. Pref.']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.division'])
        assert neighbors == set([u'fb:cell.1_aut_pref'])

    def test_read_from_json_handles_parentheses_correctly(self):
        json = {
                u'question': [],
                u'columns': [u'Urban settlements'],
                u'cells': [[u'Dzhebariki-Khaya\\n(Джебарики-Хая)'],
                          [u'South Korea (KOR)'],
                          [u'Area (km²)']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.urban_settlements'])
        assert neighbors == set([u'fb:cell.dzhebariki_khaya',
                             u'fb:cell.south_korea_kor',
                             u'fb:cell.area_km'])

        json = {
                u'question': [],
                u'columns': [u'Margin\\nof victory'],
                u'cells': [[u'−9 (67-67-68-69=271)']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.margin_of_victory'])
        assert neighbors == set([u'fb:cell._9_67_67_68_69_271'])

        json = {
                u'question': [],
                u'columns': [u'Record'],
                u'cells': [[u'4.08 m (13 ft 41⁄2 in)']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row.record'])
        assert neighbors == set([u'fb:cell.4_08_m_13_ft_41_2_in'])

    def test_read_from_json_handles_columns_with_duplicate_normalizations(self):
        json = {
                u'question': [],
                u'columns': [u'# of votes', u'% of votes'],
                u'cells': [[u'1', u'2'],
                          [u'3', u'4']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        neighbors = set(graph.neighbors[u'fb:row.row._of_votes'])
        assert neighbors == set([u'fb:cell.1', u'fb:cell.3'])
        neighbors = set(graph.neighbors[u'fb:row.row._of_votes_2'])
        assert neighbors == set([u'fb:cell.2', u'fb:cell.4'])
        neighbors = set(graph.neighbors[u'fb:cell.1'])
        assert neighbors == set([u'fb:row.row._of_votes'])

    def test_read_from_json_handles_cells_with_duplicate_normalizations(self):
        json = {
                u'question': [],
                u'columns': [u'answer'],
                u'cells': [[u'yes'], [u'yes*'], [u'yes'], [u'yes '], [u'yes*']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)

        # There are three unique text strings that all normalize to "yes", so there are three
        # fb:cell.yes entities.  Hopefully we produce them in the same order as SEMPRE does...
        assert graph.entities == [u'-1', u'0', u'1', u'fb:cell.yes', u'fb:cell.yes_2', u'fb:cell.yes_3',
                                  u'fb:row.row.answer']

    def test_get_numbers_from_tokens_works_for_arabic_numerals(self):
        tokens = [Token(x) for x in [u'7', u'1.0', u'-20']]
        numbers = TableQuestionKnowledgeGraph._get_numbers_from_tokens(tokens)
        assert numbers == [(u'7', u'7'), (u'1.000', u'1.0'), (u'-20', u'-20')]

    def test_get_numbers_from_tokens_works_for_ordinal_and_cardinal_numbers(self):
        tokens = [Token(x) for x in [u'one', u'five', u'Seventh']]
        numbers = TableQuestionKnowledgeGraph._get_numbers_from_tokens(tokens)
        assert numbers == [(u'1', u'one'), (u'5', u'five'), (u'7', u'Seventh')]

    def test_get_numbers_from_tokens_works_for_months(self):
        tokens = [Token(x) for x in [u'January', u'March', u'october']]
        numbers = TableQuestionKnowledgeGraph._get_numbers_from_tokens(tokens)
        assert numbers == [(u'1', u'January'), (u'3', u'March'), (u'10', u'october')]

    def test_get_numbers_from_tokens_works_for_units(self):
        tokens = [Token(x) for x in [u'1ghz', u'3.5mm', u'-2m/s']]
        numbers = TableQuestionKnowledgeGraph._get_numbers_from_tokens(tokens)
        assert numbers == [(u'1', u'1ghz'), (u'3.500', u'3.5mm'), (u'-2', u'-2m/s')]

    def test_get_numbers_from_tokens_works_with_magnitude_words(self):
        tokens = [Token(x) for x in [u'one', u'million', u'7', u'thousand']]
        numbers = TableQuestionKnowledgeGraph._get_numbers_from_tokens(tokens)
        assert numbers == [(u'1000000', u'one million'), (u'7000', u'7 thousand')]

    def test_get_linked_agenda_items(self):
        json = {
                u'question': [Token(x) for x in [u'where', u'is', u'mersin', u'?']],
                u'columns': [u'Name in English', u'Location'],
                u'cells': [[u'Paradeniz', u'Mersin'],
                          [u'Lake Gala', u'Edirne']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        assert graph.get_linked_agenda_items() == [u'fb:cell.mersin', u'fb:row.row.location']

    def test_get_longest_span_matching_entities(self):
        json = {
                u'question': [Token(x) for x in [u'where', u'is', u'lake', u'big', u'gala', u'?']],
                u'columns': [u'Name in English', u'Location'],
                u'cells': [[u'Paradeniz', u'Lake Big'],
                          [u'Lake Big Gala', u'Edirne']]
                }
        graph = TableQuestionKnowledgeGraph.read_from_json(json)
        assert graph._get_longest_span_matching_entities() == [u'fb:cell.lake_big_gala']




from __future__ import absolute_import
import torch

from allennlp.modules.openai_transformer import OpenaiTransformer
from allennlp.modules.scalar_mix import ScalarMix
from allennlp.modules.token_embedders.token_embedder import TokenEmbedder
from allennlp.nn.util import get_range_vector, get_device_of


class OpenaiTransformerEmbedder(TokenEmbedder):
    u"""
    Takes a byte-pair representation of a batch of sentences
    (as produced by the ``OpenaiTransformerBytePairIndexer``)
    and generates a `ScalarMix` of the corresponding contextual embeddings.



    Parameters
    ----------
    transformer: ``OpenaiTransformer``, required.
        The ``OpenaiTransformer`` module used for the embeddings.
    """
    def __init__(self,
                 transformer                   )        :
        super(OpenaiTransformerEmbedder, self).__init__()

        self._transformer = transformer
        self._scalar_mix = ScalarMix(transformer.num_output_layers, do_layer_norm=False)

    def get_output_dim(self):
        u"""
        The last dimension of the output, not the shape.
        """
        return self._transformer.embed.embedding_dim

    def forward(self, inputs              , offsets              )                :
        u"""
        Parameters
        ----------
        inputs: ``torch.Tensor``, required
            A ``(batch_size, num_timesteps)`` tensor representing the byte-pair encodings
            for the current batch.
        offsets: ``torch.Tensor``, required
            A ``(batch_size, max_sequence_length)`` tensor representing the word offsets
            for the current batch.

        Returns
        -------
        ``[torch.Tensor]``
            An embedding representation of the input sequence
            having shape ``(batch_size, sequence_length, embedding_dim)``
        """
        # pylint: disable=arguments-differ
        batch_size, num_timesteps = inputs.size()

        # the transformer "vocab" consists of the actual vocab and the
        # positional encodings. Here we want the count of just the former.
        vocab_size = self._transformer.vocab_size - self._transformer.n_ctx

        # vocab_size, vocab_size + 1, ...
        positional_encodings = get_range_vector(num_timesteps, device=get_device_of(inputs)) + vocab_size

        # Combine the inputs with positional encodings
        batch_tensor = torch.stack([
                inputs,   # (batch_size, num_timesteps)
                positional_encodings.expand(batch_size, num_timesteps)
        ], dim=-1)

        byte_pairs_mask = inputs != 0

        # Embeddings is num_output_layers x (batch_size, num_timesteps, embedding_dim)
        layer_activations = self._transformer(batch_tensor)

        # Output of scalar_mix is (batch_size, num_timesteps, embedding_dim)
        mix = self._scalar_mix(layer_activations, byte_pairs_mask)

        # These embeddings are one per byte-pair, but we want one per original _word_.
        # So we choose the embedding corresponding to the last byte pair for each word,
        # which is captured by the ``offsets`` input.
        range_vector = get_range_vector(batch_size, device=get_device_of(mix)).unsqueeze(1)
        last_byte_pair_embeddings = mix[range_vector, offsets]

        return last_byte_pair_embeddings

OpenaiTransformerEmbedder = TokenEmbedder.register(u"openai_transformer_embedder")(OpenaiTransformerEmbedder)



# pylint: disable=no-self-use,invalid-name,no-value-for-parameter



from __future__ import division
from __future__ import absolute_import
import torch

from allennlp.common.testing.model_test_case import ModelTestCase
from allennlp.nn.decoding.chu_liu_edmonds import decode_mst

class BiaffineDependencyParserTest(ModelTestCase):

    def setUp(self):
        super(BiaffineDependencyParserTest, self).setUp()
        self.set_up_model(self.FIXTURES_ROOT / u"biaffine_dependency_parser" / u"experiment.json",
                          self.FIXTURES_ROOT / u"data" / u"dependencies.conllu")

    def test_dependency_parser_can_save_and_load(self):
        self.ensure_model_can_train_save_and_load(self.param_file)


    def test_mst_decoding_can_run_forward(self):
        self.model.use_mst_decoding_for_validation = True
        self.ensure_model_can_train_save_and_load(self.param_file)


    def test_batch_predictions_are_consistent(self):
        self.ensure_batch_predictions_are_consistent()

    def test_decode_runs(self):
        self.model.eval()
        training_tensors = self.dataset.as_tensor_dict()
        output_dict = self.model(**training_tensors)
        decode_output_dict = self.model.decode(output_dict)

        assert set(decode_output_dict.keys()) == set([u'arc_loss', u'tag_loss', u'loss',
                                                      u'predicted_dependencies', u'predicted_heads',
                                                      u'words', u'pos'])

    def test_mst_respects_no_outgoing_root_edges_constraint(self):
        # This energy tensor expresses the following relation:
        # energy[i,j] = "Score that i is the head of j". In this
        # case, we have heads pointing to their children.

        # We want to construct a case that has 2 children for the ROOT node,
        # because in a typical dependency parse there should only be one
        # word which has the ROOT as it's head.
        energy = torch.Tensor([[0, 9, 5],
                               [2, 0, 4],
                               [3, 1, 0]])

        length = torch.LongTensor([3])
        heads, _ = decode_mst(energy.numpy(), length.item(), has_labels=False)

        # This is the correct MST, but not desirable for dependency parsing.
        assert list(heads) == [-1, 0, 0]
        # If we run the decoding with the model, it should enforce
        # the constraint.
        heads_model, _ = self.model._run_mst_decoding(energy.view(1, 1, 3, 3), length) # pylint: disable=protected-access
        assert heads_model.tolist()[0] == [0, 0, 1]

    def test_mst_decodes_arc_labels_with_respect_to_unconstrained_scores(self):
        energy = torch.Tensor([[0, 2, 1],
                               [10, 0, 0.5],
                               [9, 0.2, 0]]).view(1, 1, 3, 3).expand(1, 2, 3, 3).contiguous()
        # Make the score for the root label for arcs to the root token be higher - it
        # will be masked for the MST, but we want to make sure that the tags are with
        # respect to the unmasked tensor. If the masking was incorrect, we would decode all
        # zeros as the labels, because torch takes the first index in the case that all the
        # values are equal, which would be the case if the labels were calculated from
        # the masked score.
        energy[:, 1, 0, :] = 3
        length = torch.LongTensor([3])
        heads, tags = self.model._run_mst_decoding(energy, length) # pylint: disable=protected-access
        assert heads.tolist()[0] == [0, 0, 1]
        assert tags.tolist()[0] == [0, 1, 0]



# pylint: disable=invalid-name,no-self-use,protected-access

from __future__ import absolute_import
import numpy
from numpy.testing import assert_almost_equal
import torch
from torch.autograd import Variable
from torch.nn import Parameter

from allennlp.common import Params
from allennlp.common.testing.test_case import AllenNlpTestCase
from allennlp.modules.matrix_attention import LinearMatrixAttention
from allennlp.modules.matrix_attention.matrix_attention import MatrixAttention


class TestLinearMatrixAttention(AllenNlpTestCase):

    def test_can_init_dot(self):
        legacy_attention = MatrixAttention.from_params(Params({u"type": u"linear",
                                                               u"tensor_1_dim": 3,
                                                               u"tensor_2_dim": 3}))
        isinstance(legacy_attention, LinearMatrixAttention)

    def test_linear_similarity(self):
        linear = LinearMatrixAttention(3, 3)
        linear._weight_vector = Parameter(torch.FloatTensor([-.3, .5, 2.0, -1.0, 1, 1]))
        linear._bias = Parameter(torch.FloatTensor([.1]))
        output = linear(Variable(torch.FloatTensor([[[0, 0, 0], [4, 5, 6]], [[-7, -8, -9], [10, 11, 12]]])),
                        Variable(torch.FloatTensor([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]])))

        assert_almost_equal(output.data.numpy(), numpy.array([[[4.1000, 7.1000], [17.4000, 20.4000]],
                                                              [[-9.8000, -6.8000], [36.6000, 39.6000]]]),
                            decimal=2)



u"""
This is a tiny webapp for generating configuration stubs for your models.
It's very hacky and very experimental, so don't rely on it for anything important.

```
python -m allennlp.service.config_explorer
```

will launch the app on `localhost:8123` (you can specify a different port if you like).

It can also incorporate your own classes if you use the `include_package` flag:

```
python -m allennlp.service.config_explorer \
    --include-package my_library
```
"""
# pylint: disable=too-many-return-statements

from __future__ import absolute_import
from __future__ import print_function
#typing
import argparse
import logging
import sys

from flask import Flask, request, Response, jsonify, send_file
from flask_cors import CORS
from gevent.pywsgi import WSGIServer

from allennlp.common.configuration import configure, Config
from allennlp.common.util import import_submodules
from allennlp.service.server_simple import ServerError

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


def make_app(include_packages                = ())         :
    u"""
    Creates a Flask app that serves up a simple configuration wizard.
    """
    # Load modules
    for package_name in include_packages:
        import_submodules(package_name)

    app = Flask(__name__)  # pylint: disable=invalid-name

    @app.errorhandler(ServerError)
    def handle_invalid_usage(error             )            :  # pylint: disable=unused-variable
        response = jsonify(error.to_dict())
        response.status_code = error.status_code
        return response

    @app.route(u'/')
    def index()            :  # pylint: disable=unused-variable
        return Response(response=_HTML, status=200)

    @app.route(u'/debug/')
    def wizard()            :  # pylint: disable=unused-variable
        u"""
        This is just for quick iteration while developing, don't use it otherwise.
        """
        return send_file(u'config_explorer.html')

    @app.route(u'/api/config/')
    def api()            :  # pylint: disable=unused-variable
        class_name = request.args.get(u'class', u'')

        config = configure(class_name)

        if isinstance(config, Config):
            return jsonify({
                    u"className": class_name,
                    u"config": config.to_json()
            })
        else:
            return jsonify({
                    u"className": class_name,
                    u"choices": config
            })

    return app


def main(args):
    parser = argparse.ArgumentParser(description=u'Serve up a simple configuration wizard')

    parser.add_argument(u'--port', type=int, default=8123, help=u'port to serve the wizard on')

    parser.add_argument(u'--include-package',
                        type=unicode,
                        action=u'append',
                        default=[],
                        help=u'additional packages to include')

    args = parser.parse_args(args)

    app = make_app(args.include_package)
    CORS(app)

    http_server = WSGIServer((u'0.0.0.0', args.port), app)
    print("serving Config Explorer on port {args.port}")
    http_server.serve_forever()

_HTML = u"""
<!DOCTYPE html>
<html>

<head>
    <meta charset="utf-8">
    <title>AllenNLP Configuration Wizard (alpha)</title>
    <style>
        * {
            font-family: sans-serif;
        }

        .fa-times,
        .fa-plus {
            margin-top: 2px;
            color: gray;
            font-size: 12px;
        }

        .required-icon,
        .tooltip-icon,
        .subconfigure .fa {
            margin: 0 5px;
            color: gray;
            font-size: 15px;
        }

        .required-icon {
            color: red;
            position: absolute;
        }

        .tooltip-icon {
            font-size: 13px;
        }

        .name {
            border-right: thin solid darkGray;
            padding: 3px;
            margin: 5px 5px 5px 14px;
        }

        input {
            margin-left: 10px;
        }

        select {
            margin: auto 10px;
            background: white;
            font-size: 14px;
        }

        h1,
        h2 {
            -webkit-font-smoothing: antialiased;
            -moz-osx-font-smoothing: grayscale;
            font-weight: 300
        }

        h2 {
            font-size: 2em;
        }

        .annotation {
            font-size: 90%;
            margin-left: 10px;
            margin-right: 10px;
            color: #2085bc;
        }

        .prefix {
            font-size: 14px;
            margin-left: 10px;
        }

        div#rendered-json {
            margin: 10px;
            border: 1px solid black;
            font-family: monospace;
            white-space: pre;
        }

        .config,
        .list,
        .dict {
            margin-top: 5px;
        }

        .config-item {
            margin-top: 2px;
        }

        .subconfigure {
            margin-left: 5px;
            margin-right: 5px;
            cursor: pointer;
            vertical-align: middle;
            color: black;
        }

        .tippy-content {
            color: white;
        }

        .tree ul {
            margin-left: -10px;
        }

        .tree li {
            list-style-type: none;
            margin: 5px 10px;
            position: relative;
        }

        .tree li::before {
            content: "";
            position: absolute;
            top: -7px;
            left: -20px;
            border-left: 1px solid #ccc;
            border-bottom: 1px solid #ccc;
            border-radius: 0 0 0 0px;
            width: 20px;
            height: 15px;
        }

        .tree li::after {
            position: absolute;
            content: "";
            top: 8px;
            left: -20px;
            border-left: 1px solid #ccc;
            border-top: 1px solid #ccc;
            border-radius: 0px 0 0 0;
            width: 20px;
            height: 100%;
        }

        .tree li:last-child::after {
            display: none;
        }

        .tree li:last-child:before {
            border-radius: 0 0 0 5px;
        }

        ul.tree>li:first-child::before {
            display: none;
        }

        ul.tree>li:first-child::after {
            border-radius: 5px 0 0 0;
        }

        .tree li section {
            border: 1px #ccc solid;
            border-radius: 5px;
            padding: 2px 5px;
        }

        .tree li section:hover,
        .tree li section:hover+ul li section,
        .tree li section:focus,
        .tree li section:focus+ul li section {
            background: #f9f9f9;
            color: #000;
            border: 1px solid #000;
        }

        .tree li section:hover+ul li::after,
        .tree li section:focus+ul li::after,
        .tree li section:hover+ul li::before,
        .tree li section:focus+ul li::before .tree li section:hover+ul::before,
        .tree li section:focus+ul::before .tree li section:hover+ul ul::before,
        .tree li section:focus+ul ul::before {
            border-color: #000;
            /*connector color on hover*/
        }
    </style>
    <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.1.0/css/all.css" integrity="sha384-lKuwvrZot6UHsBSfcMvOkWwlCMgc0TaWr+30HWe3a4ltaBwTZhyTEggF5tJv8tbt"
        crossorigin="anonymous">
</head>

<body>
    <h2>AllenNLP Configuration Wizard (alpha)</h2>

    <div id="app"></div>

    <script crossorigin src="https://unpkg.com/react@16/umd/react.production.min.js"></script>
    <script crossorigin src="https://unpkg.com/react-dom@16/umd/react-dom.production.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/immutable/3.8.2/immutable.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/babel-standalone/6.26.0/babel.js"></script>
    <script src="https://unpkg.com/tippy.js@2.5.2/dist/tippy.all.min.js"></script>
    <script type="text/babel">
        /*
        The API returns objects that look like the following:

        ConfigItem = {
        name : str
        annotation : List[str]
        configurable : bool
        defaultValue : str
        comment : str

        Config = {
            type: str
            items: List[ConfigItem]
        }

        ApiResponse = {
            className: str
            config: Config
        } or {
            className: str
            choices: Dict[str, str]  // name -> className
        }
        */

        const baseApi = ''

        // A configItem is optional if it has a default value
        const isOptional = (configItem) => configItem.get('defaultValue') !== undefined

        // Assumption is that "allennlp.*" and "torch.*" are configurable
        // (at least, if they occur in a config file)
        const isConfigurable = (annotation) => {
            const origin = annotation.get('origin')
            return origin.startsWith('allennlp.') || origin.startsWith('torch.')
        }

        // Sometimes we don't have type annotations
        // (e.g. for torch classes)
        // and we have to best guess how to serialize an input.
        const bestGuess = x => {
            const asNumber = +x

            if (x === "true") {
                return true
            } else if (x === "false") {
                return false
            } else if (!isNaN(asNumber)) {
                return asNumber
            } else {
                // Assume string
                return x
            }
        }

        // Recursively convert the provided value to JSON
        const jsonify = (value, annotation, configurable, optional) => {
            if (!value && optional) {
                return undefined
            } else if (configurable) {
                return configToJson(value)
            } else {
                const origin = annotation.get('origin')
                const args = annotation.get('args')

                if (origin === '?') {
                    return bestGuess(value)
                }
                if (origin === 'Dict') {
                    const valueAnnotation = args.get(1)
                    const configurable = isConfigurable(valueAnnotation)

                    const dict = {}
                    let nonEmpty = false;

                    (value || Immutable.List()).forEach((entry) => {
                        const entryKey = entry.get("key")
                        const entryValue = entry.get("value")
                        if (entryKey && entryKey.length && entryValue) {
                            const valueJson = jsonify(entryValue, valueAnnotation, configurable, true)
                            if (valueJson) {
                                nonEmpty = true
                                dict[entryKey] = valueJson
                            }
                        }
                    })

                    return (nonEmpty || !optional) ? dict : undefined
                } else if (origin === 'List' || origin === 'Sequence') {
                    const [valueAnnotation] = args
                    const configurable = isConfigurable(valueAnnotation)

                    const list = (value || Immutable.List()).map(item => jsonify(item.get('value'), valueAnnotation, configurable, true))
                        .filter(x => x !== undefined)
                        .toArray()
                    return (list.length || !optional) ? list : undefined
                } else if (origin === 'int' || origin === 'float') {
                    const numeric = +value
                    return value && value.length && !isNaN(numeric) ? numeric : undefined
                } else if (origin === 'bool') {
                    return { 'true': true, 'false': false }[value]
                } else if (origin === 'str') {
                    const keep = (value && value.length) || !optional
                    return keep ? (value || '') : undefined
                } else {
                    console.log("unknown type " + annotation.toJS())
                    return undefined
                }
            }
        }

        const configToJson = (config) => {

            let blob = {}

            if (config) {
                const type = config.get('type')
                if (type) {
                    blob['type'] = type
                }

                config.get('items').forEach((item) => {
                    const value = item.get('value')
                    const annotation = item.get('annotation')
                    const configurable = item.get('configurable')
                    const optional = item.get('defaultValue') !== undefined

                    const json = jsonify(value, annotation, configurable, optional)

                    if (json !== undefined) {
                        const name = item.get('name')
                        blob[name] = json
                    }
                })
            }

            return blob
        }

        // Each node in the config tree will be marked complete if either
        // * it's optional, or
        // * it's a leaf and has a value, or
        // * all of its children are complete
        //
        // This means it's a bottom up concept. This is the one place
        // where immutability makes our lives a little trickier.
        const markComplete = (node) => {
            const optional = node.get('defaultValue') !== undefined
            const origin = node.getIn(['annotation', 'origin'])
            const value = node.get('value')

            if (node.get('configurable')) {
                if (value) {
                    const items = node.getIn(['value', 'items']).map(markComplete)
                    return node.set('completed', optional || items.every(item => item.get('completed')))
                        .setIn(['value', 'items'], items)
                } else {
                    return node.set('completed', optional)
                }
            } else if (Immutable.List.isList(value)) {
                const items = value.map(markComplete)
                return node.set('completed', optional || items.every(item => item.get('completed')))
                    .set('value', items)
            } else {
                const completed = optional || value
                return node.set('completed', completed)
            }
        }

        // Top-level component
        class App extends React.Component {
            constructor() {
                super()

                this.state = {
                    data: Immutable.Map()
                }

                this.setData = this.setData.bind(this)
            }

            setData(fn) {
                const { data } = this.state
                const newData = markComplete(fn(data))
                return this.setState({ data: newData })
            }

            componentDidMount() {
                // Fetch the top level configuration
                fetch(`${baseApi}/api/config/`)
                    .then(res => res.json())
                    .then(({ config }) => {
                        const data = Immutable.Map({ configurable: true, value: Immutable.fromJS(config) })
                        this.setState({ data: data })
                    })
            }

            render() {
                const config = this.state.data.get('value')
                return (
                    <div class="wizard">
                        <div class="tree">
                            <Config path={Immutable.List(['value'])} setData={this.setData} config={config} />
                        </div>
                        <JsonBox config={config} />
                    </div>
                )
            }
        }

        // Component that displays the JSON-rendered config file
        const JsonBox = ({ config }) => {
            const json = configToJson(config)

            const selectAll = () => {
                window.getSelection()
                    .selectAllChildren(document.getElementById('rendered-json'))
            }

            return (
                <div id="rendered-json" onClick={selectAll}>
                    {JSON.stringify(json, null, 4)}
                </div>
            )
        }

        // Component that renders either
        // * the entire config tree, or
        // * the params for a configurable class
        const Config = ({ path, config, setData }) => {
            if (!config) {
                return null
            }

            const type = config.get('type')
            const items = config.get('items')

            const renderedType = type ? (
                <li class="config-item">
                    <section>
                        <span class="name">type</span>
                        <span>{type}</span>
                    </section>
                </li>
            ) : null

            const renderedItems = items.map((item, idx) =>
                <ConfigItem path={path.push('items', idx)} item={item} setData={setData} />
            )

            return (
                <ul class="config">
                    {renderedType}
                    {renderedItems}
                </ul>
            )
        }

        // A ConfigItem will consist of a name (the key in its param dict)
        // and a child that could be a text input, a list input, a dict input
        // or an entire configuration. This function renders the appropriate one.
        const renderChild = (path, item, setData) => {
            const annotation = item.get('annotation')
            const configurable = item.get('configurable')
            const origin = annotation.get('origin')
            const args = annotation.get('args')

            if (configurable) {
                return <Configurator path={path} item={item} setData={setData} />
            } else if (origin === 'Dict') {
                return <Dict path={path} item={item} setData={setData} />
            } else if (origin === 'List' || origin == 'Sequence' || (origin == 'Tuple' && args.size == 2 && args.get(2) === '...')) {
                return <List path={path} item={item} setData={setData} />
            } else {
                return <TextInput path={path} item={item} setData={setData} />
            }
        }

        class Tooltip extends React.Component {
            componentDidMount() {
                tippy('.tooltip')
            }

            render() {
                return <span class="tooltip tooltip-icon " title={this.props.title} tabIndex="-1"><i class="fas fa-question-circle"></i></span>
            }
        }

        class RequiredWarning extends React.Component {
            componentDidMount() {
                tippy('.tooltip')
            }

            render() {
                return <span class="tooltip required-icon " title={this.props.title} tabIndex="-1"><i class="fas fa-exclamation"></i></span>
            }
        }

        // Represent an annotation as text
        const renderAnnotation = (annotation) => {
            const origin = annotation.get('origin')
            const args = annotation.get('args')

            if (args) {
                return `${origin}[${args.map(renderAnnotation).join(', ')}]`
            } else {
                return origin
            }
        }

        // single input boxes and configuration buttons stay on the parent line
        function showItemInline(item, inGroup) {
            const configurable = item.get('configurable')
            if (configurable) {
                const choices = item.get('choices')
                const config = item.get('value')
                // configurators with no choices nor config are inlined
                // if they are open with choices/config, then we dont inline
                return (!choices && !config)
            }
            const annotation = item.get('annotation')
            const origin = annotation.get('origin')
            switch(origin){
                case 'Dict':
                case 'List':
                case 'Sequence': {
                    // complex data is never inlined
                    return false
                }
                case 'Tuple': {
                    const args = annotation.get('args')
                    // tuples treated as lists are not inlined
                    // tuples with 2 items are inlined
                    return args.size == 2 && args.get(2) === '...'
                }
            }
            // simple input is inlined
            return true;
        }

        // One of the key-value pairs that makes up a Config.
        const ConfigItem = ({ path, item, setData }) => {
            const name = item.get('name')
            const annotation = item.get('annotation')
            const configurable = item.get('configurable')
            const defaultValue = item.get('defaultValue')
            const comment = item.get('comment')
            const completed = item.get('completed')

            const classNames = [
                'config-item',
                defaultValue === undefined ? 'required' : 'optional',
                completed ? 'complete' : 'incomplete'
            ].join(' ')

            const showRequired = defaultValue === undefined && !completed
            const showInline = showItemInline(item, false)
            const tooltip = comment ? <Tooltip title={comment} /> : null
            const isRequired = showRequired ? <RequiredWarning title="Required" /> : null
            const renderedAnnotation = <span class="annotation">({renderAnnotation(annotation)})</span>

            return (
                <li className={classNames}>
                    <section>
                        {isRequired}
                        <span class="name">{name} {tooltip}</span>
                        {renderedAnnotation}
                        {showInline ? renderChild(path, item, setData) : null}
                    </section>
                    {!showInline ? <ul>{renderChild(path, item, setData)}</ul> : null}
                </li>
            )
        }

        // Find the longest common prefix
        const sharedPrefix = (s1, s2) => {
            for (let i = 0; i < s1.length; i++) {
                if (s1[i] !== s2[i]) {
                    return s1.slice(0, i)
                }
            }
            // Otherwise, all of s1
            return s1
        }

        const commonPrefix = (strings) => {
            if (strings.size == 1) {
                // Special case, return up to last .
                const [string] = strings
                const idx = string.lastIndexOf(".")
                return string.slice(0, idx + 1)
            } else {
                return strings.reduce(sharedPrefix)
            }
        }

        const Configurator = ({ path, item, setData }) => {
            const annotation = item.get('annotation')
            const className = annotation.get('origin')

            // User-supplied
            const choices = item.get('choices')
            const choice = item.get('choice')
            const config = item.get('value')

            let prefix = choices ? commonPrefix(choices) : null

            const getChoices = () => {
                fetch(`${baseApi}/api/config/?class=${className}`)
                    .then(res => res.json())
                    .then(({ config, choices }) => {
                        if (choices) {
                            setData(oldConfig => oldConfig.setIn(path.push('choices'), Immutable.List(choices)))
                        } else {
                            setData(oldConfig => oldConfig.setIn(path.push('value'), Immutable.fromJS(config)))
                        }
                    })
            }

            const select = (evt) => {
                const choice = evt.target.value

                fetch(`${baseApi}/api/config/?class=${choice}`)
                    .then(res => res.json())
                    .then(({ config }) => {
                        setData(rootConfig => rootConfig.setIn(path.push('choice'), choice)
                            .setIn(path.push('value'), Immutable.fromJS(config)))
                    })
            }

            const remove = () => {
                setData(config => config.deleteIn(path.push('choices'))
                    .deleteIn(path.push('value'))
                    .deleteIn(path.push('choice')))
            }

            const configureButton = (choices || config) ? null : <button class="subconfigure" onClick={getChoices}><i class="fas fa-wrench" /> Configure</button>
            const deleteButton = choices ? <button onClick={remove}><i class="fas fa-times" /></button> : null
            const renderedValue = config ? <Config path={path.push('value')} config={config} setData={setData} /> : null
            const choicesDropdown = choices ? (
                <span>
                    <span class="prefix">{prefix}</span>
                    <select value={choice} onChange={select}>
                        {choice ? null : (<option value=""></option>)}
                        {choices.map(subclass => <option value={subclass}>{subclass.slice(prefix ? prefix.length : 0)}</option>)}
                    </select>
                    {deleteButton}
                </span>
            ) : null
            const options = (choicesDropdown || renderedValue) ? (
                <li class="choices-dropdown">
                    {choicesDropdown}
                    {renderedValue}
                </li>
            ) : null

            return (
                <span>
                    {configureButton}
                    {options}
                </span>
            )
        }

        const List = ({ path, item, setData }) => {
            const [valueType] = item.get('annotation').get('args')
            const configurable = isConfigurable(valueType)
            const items = item.get('value') || Immutable.List()

            const addItem = () => {
                // We use `counter` to give every list item a unique key.
                const counter = item.get('counter') || 1

                const newItem = Immutable.Map({
                    "annotation": valueType,
                    "configurable": configurable,
                    "itemKey": path.push(counter).join('-')
                })

                setData(config => config.setIn(path.push('value'), items.push(newItem))
                    .setIn(path.push('counter'), counter + 1))
            }

            const removeItem = idx => () => {
                setData(config => config.setIn(path.push('value'), items.delete(idx)))
            }

            const renderedItems = items.map((item, idx) => {
                const showInline = showItemInline(item, true)
                return (
                    <li class="list-item" key={item.get('itemKey')}>
                        <section>
                            <button onClick={removeItem(idx)}><i class="fas fa-times" /></button>
                            {showInline ? renderChild(path.push('value', idx), item, setData) : null}
                        </section>
                        {!showInline ? <ul>{renderChild(path.push('value', idx), item, setData)}</ul> : null}
                    </li>
                )
            })

            return (
                <div class="list">
                    {renderedItems}
                    <li><button onClick={addItem}><i class="fas fa-plus" /></button></li>
                </div>
            )
        }

        const Dict = ({ path, item, setData }) => {
            const [keyType, valueType] = item.get('annotation').get('args')
            const configurable = isConfigurable(valueType)
            const items = item.get('value') || Immutable.List()

            const addItem = () => {
                const counter = item.get('counter') || 1

                const newItem = Immutable.Map({
                    "annotation": valueType,
                    "configurable": configurable,
                    "itemKey": path.push(counter).join('-')
                })

                setData(config => config.setIn(path.push('value'), items.push(newItem))
                    .setIn(path.push('counter'), counter + 1))
            }

            const removeItem = idx => () => {
                setData(config => config.setIn(path.push('value'), items.delete(idx)))
            }

            const renderedItems = items.map((item, idx) => {
                const showInline = showItemInline(item, true)
                return (
                    <li class="dict-item" key={item.get('itemKey')}>
                        <section>
                            <button onClick={removeItem(idx)}><i class="fas fa-times" /></button>
                            <TextInput path={path.push('value', idx)} fieldName="key" item={item} setData={setData} />
                            {showInline ? renderChild(path.push('value', idx), item, setData) : null}
                        </section>
                        {!showInline ? <ul>{renderChild(path.push('value', idx), item, setData)}</ul> : null}
                    </li>
                )
            })

            return (
                <div class="dict">
                    {renderedItems}
                    <li><button onClick={addItem}><i class="fas fa-plus" /></button></li>
                </div>
            )
        }

        const TextInput = ({ path, item, setData, fieldName }) => {
            fieldName = fieldName || 'value'

            const onChange = evt => {
                setData(config => config.setIn(path.push(fieldName), evt.target.value))
            }
            const defaultValue = item.get('defaultValue')
            const placeholder = defaultValue === undefined ? 'REQUIRED' : `default: ${defaultValue}`

            return (
                <span>
                    <input type="text" value={item.get(fieldName)} placeholder={placeholder} onChange={onChange} />
                </span>
            )
        }

        ReactDOM.render(<App />, document.getElementById("app"))
    </script>
</body>

</html>
"""


if __name__ == u"__main__":
    main(sys.argv[1:])



# pylint: disable=invalid-name,no-self-use



from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
import os
import sys
from io import StringIO

from allennlp.commands import main
from allennlp.common.testing import AllenNlpTestCase
from io import open


class TestConfigure(AllenNlpTestCase):

    def test_other_modules(self):
        # Create a new package in a temporary dir
        packagedir = self.TEST_DIR / u'configuretestpackage'
        packagedir.mkdir()  # pylint: disable=no-member
        (packagedir / u'__init__.py').touch()  # pylint: disable=no-member

        # And add that directory to the path
        sys.path.insert(0, unicode(self.TEST_DIR))

        # Write out a duplicate predictor there, but registered under a different name.
        from allennlp.predictors import bidaf
        with open(bidaf.__file__) as f:
            code = f.read().replace(u"""@Predictor.register('machine-comprehension')""",
                                    u"""@Predictor.register('configure-test-predictor')""")

        with open(os.path.join(packagedir, u'predictor.py'), u'w') as f:
            f.write(code)

        # Capture stdout
        stdout_saved = sys.stdout
        stdout_captured = StringIO()
        sys.stdout = stdout_captured

        sys.argv = [u"run.py",      # executable
                    u"configure",     # command
                    u"configuretestpackage.predictor.BidafPredictor"]

        main()
        output = stdout_captured.getvalue()
        assert u"configure-test-predictor" in output

        sys.stdout = stdout_saved

        sys.path.remove(unicode(self.TEST_DIR))



# pylint: disable=no-self-use,invalid-name



from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
import torch

import pytest
import numpy

from allennlp.common.testing import AllenNlpTestCase
from allennlp.common.checks import ConfigurationError
from allennlp.modules import ScalarMix


class TestScalarMix(AllenNlpTestCase):
    def test_scalar_mix_can_run_forward(self):
        mixture = ScalarMix(3)
        tensors = [torch.randn([3, 4, 5]) for _ in range(3)]
        for k in range(3):
            mixture.scalar_parameters[k].data[0] = 0.1 * (k + 1)
        mixture.gamma.data[0] = 0.5
        result = mixture(tensors)

        weights = [0.1, 0.2, 0.3]
        normed_weights = numpy.exp(weights) / numpy.sum(numpy.exp(weights))
        expected_result = sum(normed_weights[k] * tensors[k].data.numpy() for k in range(3))
        expected_result *= 0.5
        numpy.testing.assert_almost_equal(expected_result, result.data.numpy())

    def test_scalar_mix_throws_error_on_incorrect_number_of_inputs(self):
        mixture = ScalarMix(3)
        tensors = [torch.randn([3, 4, 5]) for _ in range(5)]
        with pytest.raises(ConfigurationError):
            _ = mixture(tensors)

    def test_scalar_mix_layer_norm(self):
        mixture = ScalarMix(3, do_layer_norm=u'scalar_norm_reg')

        tensors = [torch.randn([3, 4, 5]) for _ in range(3)]
        numpy_mask = numpy.ones((3, 4), dtype=u'int32')
        numpy_mask[1, 2:] = 0
        mask = torch.from_numpy(numpy_mask)

        weights = [0.1, 0.2, 0.3]
        for k in range(3):
            mixture.scalar_parameters[k].data[0] = weights[k]
        mixture.gamma.data[0] = 0.5
        result = mixture(tensors, mask)

        normed_weights = numpy.exp(weights) / numpy.sum(numpy.exp(weights))
        expected_result = numpy.zeros((3, 4, 5))
        for k in range(3):
            mean = numpy.mean(tensors[k].data.numpy()[numpy_mask == 1])
            std = numpy.std(tensors[k].data.numpy()[numpy_mask == 1])
            normed_tensor = (tensors[k].data.numpy() - mean) / (std + 1E-12)
            expected_result += (normed_tensor * normed_weights[k])
        expected_result *= 0.5

        numpy.testing.assert_almost_equal(expected_result, result.data.numpy())




from __future__ import absolute_import
#typing

#overrides

from allennlp.data.tokenizers.token import Token
from allennlp.data.tokenizers.tokenizer import Tokenizer


class CharacterTokenizer(Tokenizer):

    def __init__(self,
                 byte_encoding      = None,
                 lowercase_characters       = False,
                 start_tokens            = None,
                 end_tokens            = None)        :
        self._byte_encoding = byte_encoding
        self._lowercase_characters = lowercase_characters
        self._start_tokens = start_tokens or []
        # We reverse the tokens here because we're going to insert them with `insert(0)` later;
        # this makes sure they show up in the right order.
        self._start_tokens.reverse()
        self._end_tokens = end_tokens or []

    #overrides
    def batch_tokenize(self, texts           )                     :
        return [self.tokenize(text) for text in texts]

    #overrides
    def tokenize(self, text     )               :
        if self._lowercase_characters:
            text = text.lower()
        if self._byte_encoding is not None:
            # We add 1 here so that we can still use 0 for masking, no matter what bytes we get out
            # of this.
            tokens = [Token(text_id=c + 1) for c in text.encode(self._byte_encoding)]
        else:
            tokens = [Token(t) for t in list(text)]
        for start_token in self._start_tokens:
            if isinstance(start_token, int):
                token = Token(text_id=start_token, idx=0)
            else:
                token = Token(text=start_token, idx=0)
            tokens.insert(0, token)
        for end_token in self._end_tokens:
            if isinstance(end_token, int):
                token = Token(text_id=end_token, idx=0)
            else:
                token = Token(text=end_token, idx=0)
            tokens.append(token)
        return tokens

CharacterTokenizer = Tokenizer.register(u"character")(CharacterTokenizer)





from __future__ import with_statement
from __future__ import absolute_import
#typing
import json
import logging

#overrides

from allennlp.common.file_utils import cached_path
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.fields import Field, TextField, LabelField, MetadataField
from allennlp.data.instance import Instance
from allennlp.data.token_indexers import SingleIdTokenIndexer, TokenIndexer
from allennlp.data.tokenizers import Tokenizer, WordTokenizer
from io import open

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class SnliReader(DatasetReader):
    u"""
    Reads a file from the Stanford Natural Language Inference (SNLI) dataset.  This data is
    formatted as jsonl, one json-formatted instance per line.  The keys in the data are
    "gold_label", "sentence1", and "sentence2".  We convert these keys into fields named "label",
    "premise" and "hypothesis", along with a metadata field containing the tokenized strings of the
    premise and hypothesis.

    Parameters
    ----------
    tokenizer : ``Tokenizer``, optional (default=``WordTokenizer()``)
        We use this ``Tokenizer`` for both the premise and the hypothesis.  See :class:`Tokenizer`.
    token_indexers : ``Dict[str, TokenIndexer]``, optional (default=``{"tokens": SingleIdTokenIndexer()}``)
        We similarly use this for both the premise and the hypothesis.  See :class:`TokenIndexer`.
    """

    def __init__(self,
                 tokenizer            = None,
                 token_indexers                          = None,
                 lazy       = False)        :
        super(SnliReader, self).__init__(lazy)
        self._tokenizer = tokenizer or WordTokenizer()
        self._token_indexers = token_indexers or {u'tokens': SingleIdTokenIndexer()}

    #overrides
    def _read(self, file_path     ):
        # if `file_path` is a URL, redirect to the cache
        file_path = cached_path(file_path)

        with open(file_path, u'r') as snli_file:
            logger.info(u"Reading SNLI instances from jsonl dataset at: %s", file_path)
            for line in snli_file:
                example = json.loads(line)

                label = example[u"gold_label"]
                if label == u'-':
                    # These were cases where the annotators disagreed; we'll just skip them.  It's
                    # like 800 out of 500k examples in the training data.
                    continue

                premise = example[u"sentence1"]
                hypothesis = example[u"sentence2"]

                yield self.text_to_instance(premise, hypothesis, label)

    #overrides
    def text_to_instance(self,  # type: ignore
                         premise     ,
                         hypothesis     ,
                         label      = None)            :
        # pylint: disable=arguments-differ
        fields                   = {}
        premise_tokens = self._tokenizer.tokenize(premise)
        hypothesis_tokens = self._tokenizer.tokenize(hypothesis)
        fields[u'premise'] = TextField(premise_tokens, self._token_indexers)
        fields[u'hypothesis'] = TextField(hypothesis_tokens, self._token_indexers)
        if label:
            fields[u'label'] = LabelField(label)

        metadata = {u"premise_tokens": [x.text for x in premise_tokens],
                    u"hypothesis_tokens": [x.text for x in hypothesis_tokens]}
        fields[u"metadata"] = MetadataField(metadata)
        return Instance(fields)

SnliReader = DatasetReader.register(u"snli")(SnliReader)



u"""
A ``TextField`` represents a string of text, the kind that you might want to represent with
standard word vectors, or pass through an LSTM.
"""

from __future__ import absolute_import
#typing
import textwrap

#overrides
from spacy.tokens import Token as SpacyToken
import torch

from allennlp.common.checks import ConfigurationError
from allennlp.data.fields.sequence_field import SequenceField
from allennlp.data.tokenizers.token import Token
from allennlp.data.token_indexers.token_indexer import TokenIndexer
from allennlp.data.vocabulary import Vocabulary
from allennlp.nn import util


class TextField(SequenceField):
    u"""
    This ``Field`` represents a list of string tokens.  Before constructing this object, you need
    to tokenize raw strings using a :class:`~allennlp.data.tokenizers.tokenizer.Tokenizer`.

    Because string tokens can be represented as indexed arrays in a number of ways, we also take a
    dictionary of :class:`~allennlp.data.token_indexers.token_indexer.TokenIndexer`
    objects that will be used to convert the tokens into indices.
    Each ``TokenIndexer`` could represent each token as a single ID, or a list of character IDs, or
    something else.

    This field will get converted into a dictionary of arrays, one for each ``TokenIndexer``.  A
    ``SingleIdTokenIndexer`` produces an array of shape (num_tokens,), while a
    ``TokenCharactersIndexer`` produces an array of shape (num_tokens, num_characters).
    """
    def __init__(self, tokens             , token_indexers                         )        :
        self.tokens = tokens
        self._token_indexers = token_indexers
        self._indexed_tokens = None
        self._indexer_name_to_indexed_token = None

        if not all([isinstance(x, (Token, SpacyToken)) for x in tokens]):
            raise ConfigurationError(u"TextFields must be passed Tokens. "
                                     u"Found: {} with types {}.".format(tokens, [type(x) for x in tokens]))

    #overrides
    def count_vocab_items(self, counter                           ):
        for indexer in list(self._token_indexers.values()):
            for token in self.tokens:
                indexer.count_vocab_items(token, counter)

    #overrides
    def index(self, vocab            ):
        token_arrays                       = {}
        indexer_name_to_indexed_token                       = {}
        for indexer_name, indexer in list(self._token_indexers.items()):
            token_indices = indexer.tokens_to_indices(self.tokens, vocab, indexer_name)
            token_arrays.update(token_indices)
            indexer_name_to_indexed_token[indexer_name] = list(token_indices.keys())
        self._indexed_tokens = token_arrays
        self._indexer_name_to_indexed_token = indexer_name_to_indexed_token

    #overrides
    def get_padding_lengths(self)                  :
        u"""
        The ``TextField`` has a list of ``Tokens``, and each ``Token`` gets converted into arrays by
        (potentially) several ``TokenIndexers``.  This method gets the max length (over tokens)
        associated with each of these arrays.
        """
        # Our basic outline: we will iterate over `TokenIndexers`, and aggregate lengths over tokens
        # for each indexer separately.  Then we will combine the results for each indexer into a single
        # dictionary, resolving any (unlikely) key conflicts by taking a max.
        lengths = []
        if self._indexed_tokens is None:
            raise ConfigurationError(u"You must call .index(vocabulary) on a "
                                     u"field before determining padding lengths.")

        # Each indexer can return a different sequence length, and for indexers that return
        # multiple arrays each can have a different length.  We'll keep track of them here.
        for indexer_name, indexer in list(self._token_indexers.items()):
            indexer_lengths = {}

            for indexed_tokens_key in self._indexer_name_to_indexed_token[indexer_name]:
                # This is a list of dicts, one for each token in the field.
                token_lengths = [indexer.get_padding_lengths(token)
                                 for token in self._indexed_tokens[indexed_tokens_key]]
            if not token_lengths:
                # This is a padding edge case and occurs when we want to pad a ListField of
                # TextFields. In order to pad the list field, we need to be able to have an
                # _empty_ TextField, but if this is the case, token_lengths will be an empty
                # list, so we add the default empty padding dictionary to the list instead.
                token_lengths = [{}]
            # Iterate over the keys and find the maximum token length.
            # It's fine to iterate over the keys of the first token since all tokens have the same keys.
            for key in token_lengths[0]:
                indexer_lengths[key] = max(x[key] if key in x else 0 for x in token_lengths)
            lengths.append(indexer_lengths)

        indexer_sequence_lengths = dict((key, len(val)) for key, val in list(self._indexed_tokens.items()))
        # Get the padding lengths for sequence lengths.
        if len(set(indexer_sequence_lengths.values())) == 1:
            # This is the default case where all indexers return the same length.
            # Keep the existing 'num_tokens' key for backward compatibility with existing config files.
            padding_lengths = {u'num_tokens': list(indexer_sequence_lengths.values())[0]}
        else:
            # The indexers return different lengths.
            padding_lengths = indexer_sequence_lengths

        # Get all keys which have been used for padding for each indexer and take the max if there are duplicates.
        padding_keys = set([key for d in lengths for key in list(d.keys())])
        for padding_key in padding_keys:
            padding_lengths[padding_key] = max(x[padding_key] if padding_key in x else 0 for x in lengths)
        return padding_lengths

    #overrides
    def sequence_length(self)       :
        return len(self.tokens)

    #overrides
    def as_tensor(self,
                  padding_lengths                ,
                  cuda_device      = -1)                           :
        tensors = {}
        num_tokens = padding_lengths.get(u'num_tokens')
        for indexer_name, indexer in list(self._token_indexers.items()):
            if num_tokens is None:
                # The indexers return different lengths.
                # Get the desired_num_tokens for this indexer.
                desired_num_tokens = dict((
                        indexed_tokens_key, padding_lengths[indexed_tokens_key])
                        for indexed_tokens_key in self._indexer_name_to_indexed_token[indexer_name])
            else:
                desired_num_tokens = {indexer_name: num_tokens}

            indices_to_pad = dict((indexed_tokens_key, self._indexed_tokens[indexed_tokens_key])
                              for indexed_tokens_key in self._indexer_name_to_indexed_token[indexer_name])
            padded_array = indexer.pad_token_sequence(indices_to_pad,
                                                      desired_num_tokens, padding_lengths)
            # We use the key of the indexer to recognise what the tensor corresponds to within the
            # field (i.e. the result of word indexing, or the result of character indexing, for
            # example).
            # TODO(mattg): we might someday have a TokenIndexer that needs to use something other
            # than a LongTensor here, and it's not clear how to signal that.  Maybe we'll need to
            # add a class method to TokenIndexer to tell us the type?  But we can worry about that
            # when there's a compelling use case for it.
            indexer_tensors = dict((key, torch.LongTensor(array)) for key, array in list(padded_array.items()))
            if cuda_device > -1:
                for key in list(indexer_tensors.keys()):
                    indexer_tensors[key] = indexer_tensors[key].cuda(cuda_device)
            tensors.update(indexer_tensors)
        return tensors

    #overrides
    def empty_field(self):
        # pylint: disable=protected-access
        text_field = TextField([], self._token_indexers)
        text_field._indexed_tokens = {}
        text_field._indexer_name_to_indexed_token = {}
        for indexer_name, indexer in list(self._token_indexers.items()):
            array_keys = indexer.get_keys(indexer_name)
            for key in array_keys:
                text_field._indexed_tokens[key] = []
            text_field._indexer_name_to_indexed_token[indexer_name] = array_keys
        return text_field

    #overrides
    def batch_tensors(self, tensor_list                               )                           :
        # pylint: disable=no-self-use
        # This is creating a dict of {token_indexer_key: batch_tensor} for each token indexer used
        # to index this field.
        return util.batch_tensor_dicts(tensor_list)

    def __str__(self)       :
        indexers = dict((name, indexer.__class__.__name__) for name, indexer in list(self._token_indexers.items()))

        # Double tab to indent under the header.
        formatted_text = u"".join([u"\t\t" + text + u"\n"
                                  for text in textwrap.wrap(repr(self.tokens), 100)])
        return "TextField of length {self.sequence_length()} with "\
               "text: \n {formatted_text} \t\tand TokenIndexers : {indexers}"



u"""
A :class:`~allennlp.data.dataset_readers.dataset_reader.DatasetReader`
reads a file and converts it to a
:class:`~allennlp.data.dataset.Dataset`.
The various subclasses know how to read specific filetypes
and produce datasets in the formats required by specific models.
"""

# pylint: disable=line-too-long
# from allennlp.data.dataset_readers.atis import AtisDatasetReader
# from allennlp.data.dataset_readers.ccgbank import CcgBankDatasetReader
# from allennlp.data.dataset_readers.conll2003 import Conll2003DatasetReader
# from allennlp.data.dataset_readers.ontonotes_ner import OntonotesNamedEntityRecognition
# from allennlp.data.dataset_readers.coreference_resolution import ConllCorefReader, WinobiasReader
# from allennlp.data.dataset_readers.dataset_reader import DatasetReader
# from allennlp.data.dataset_readers.language_modeling import LanguageModelingReader
# from allennlp.data.dataset_readers.nlvr import NlvrDatasetReader
# from allennlp.data.dataset_readers.penn_tree_bank import PennTreeBankConstituencySpanDatasetReader
# from allennlp.data.dataset_readers.reading_comprehension import SquadReader, TriviaQaReader
# from allennlp.data.dataset_readers.semantic_role_labeling import SrlReader
# from allennlp.data.dataset_readers.seq2seq import Seq2SeqDatasetReader
# from allennlp.data.dataset_readers.sequence_tagging import SequenceTaggingDatasetReader
# from allennlp.data.dataset_readers.snli import SnliReader
# from allennlp.data.dataset_readers.universal_dependencies import UniversalDependenciesDatasetReader
# from allennlp.data.dataset_readers.stanford_sentiment_tree_bank import (
#         StanfordSentimentTreeBankDatasetReader)
# from allennlp.data.dataset_readers.wikitables import WikiTablesDatasetReader
# from allennlp.data.dataset_readers.quora_paraphrase import QuoraParaphraseDatasetReader



# pylint: disable=no-self-use,invalid-name



from __future__ import division
from __future__ import absolute_import
from allennlp.common.testing import AllenNlpTestCase
from allennlp.models.archival import load_archive
from allennlp.predictors import Predictor


class TestBiaffineDependencyParser(AllenNlpTestCase):
    def test_uses_named_inputs(self):
        inputs = {
                u"sentence": u"Please could you parse this sentence?",
        }

        archive = load_archive(self.FIXTURES_ROOT / u'biaffine_dependency_parser'
                               / u'serialization' / u'model.tar.gz')
        predictor = Predictor.from_archive(archive, u'biaffine-dependency-parser')

        result = predictor.predict_json(inputs)

        words = result.get(u"words")
        predicted_heads = result.get(u"predicted_heads")
        assert len(predicted_heads) == len(words)

        predicted_dependencies = result.get(u"predicted_dependencies")
        assert len(predicted_dependencies) == len(words)
        assert isinstance(predicted_dependencies, list)
        assert all(isinstance(x, unicode) for x in predicted_dependencies)

        assert result.get(u"loss") is not None
        assert result.get(u"arc_loss") is not None
        assert result.get(u"tag_loss") is not None

        hierplane_tree = result.get(u"hierplane_tree")
        hierplane_tree.pop(u"nodeTypeToStyle")
        hierplane_tree.pop(u"linkToPosition")
        # pylint: disable=line-too-long,bad-continuation
        assert result.get(u"hierplane_tree") == {u'text': u'Please could you parse this sentence ?',
                                                u'root': {u'word': u'Please', u'nodeType': u'det', u'attributes': [u'UH'], u'link': u'det', u'spans': [{u'start': 0, u'end': 7}],
                                                    u'children': [
                                                            {u'word': u'could', u'nodeType': u'nummod', u'attributes': [u'MD'], u'link': u'nummod', u'spans': [{u'start': 7, u'end': 13}]},
                                                            {u'word': u'you', u'nodeType': u'nummod', u'attributes': [u'PRP'], u'link': u'nummod', u'spans': [{u'start': 13, u'end': 17}]},
                                                            {u'word': u'parse', u'nodeType': u'nummod', u'attributes': [u'VB'], u'link': u'nummod', u'spans': [{u'start': 17, u'end': 23}]},
                                                            {u'word': u'this', u'nodeType': u'nummod', u'attributes': [u'DT'], u'link': u'nummod', u'spans': [{u'start': 23, u'end': 28}]},
                                                            {u'word': u'sentence', u'nodeType': u'nummod', u'attributes':[u'NN'], u'link': u'nummod', u'spans': [{u'start': 28, u'end': 37}]},
                                                            {u'word': u'?', u'nodeType': u'nummod', u'attributes': [u'.'], u'link': u'nummod', u'spans': [{u'start': 37, u'end': 39}]}
                                                            ]
                                                        }
                                               }
        # pylint: enable=line-too-long,bad-continuation
    def test_batch_prediction(self):
        inputs = [
                {
                        u"sentence": u"What kind of test succeeded on its first attempt?",
                },
                {
                        u"sentence": u"What kind of test succeeded on its first attempt at batch processing?",
                }
        ]

        archive = load_archive(self.FIXTURES_ROOT / u'biaffine_dependency_parser'
                               / u'serialization' / u'model.tar.gz')
        predictor = Predictor.from_archive(archive, u'biaffine-dependency-parser')

        results = predictor.predict_batch_json(inputs)
        assert len(results) == 2

        for result in results:
            sequence_length = len(result.get(u"words"))
            predicted_heads = result.get(u"predicted_heads")
            assert len(predicted_heads) == sequence_length

            predicted_dependencies = result.get(u"predicted_dependencies")
            assert len(predicted_dependencies) == sequence_length
            assert isinstance(predicted_dependencies, list)
            assert all(isinstance(x, unicode) for x in predicted_dependencies)



# pylint: disable=no-self-use,invalid-name



from __future__ import division
from __future__ import with_statement
from __future__ import absolute_import
import gzip
import warnings

import numpy
import pytest
import torch
from io import open
try:
    from itertools import izip
except:
    izip = zip

with warnings.catch_warnings():
    warnings.filterwarnings(u"ignore", category=FutureWarning)
    import h5py

from allennlp.common import Params
from allennlp.common.checks import ConfigurationError
from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Vocabulary
from allennlp.modules.token_embedders.embedding import (Embedding,
                                                        _read_pretrained_embeddings_file,
                                                        EmbeddingsTextFile,
                                                        format_embeddings_file_uri,
                                                        parse_embeddings_file_uri)


class TestEmbedding(AllenNlpTestCase):
    # pylint: disable=protected-access
    def test_get_embedding_layer_uses_correct_embedding_dim(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace(u'word1')
        vocab.add_token_to_namespace(u'word2')
        embeddings_filename = unicode(self.TEST_DIR / u"embeddings.gz")
        with gzip.open(embeddings_filename, u'wb') as embeddings_file:
            embeddings_file.write(u"word1 1.0 2.3 -1.0\n".encode(u'utf-8'))
            embeddings_file.write(u"word2 0.1 0.4 -4.0\n".encode(u'utf-8'))
        embedding_weights = _read_pretrained_embeddings_file(embeddings_filename, 3, vocab)
        assert tuple(embedding_weights.size()) == (4, 3)  # 4 because of padding and OOV
        with pytest.raises(ConfigurationError):
            _read_pretrained_embeddings_file(embeddings_filename, 4, vocab)

    def test_forward_works_with_projection_layer(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace(u'the')
        vocab.add_token_to_namespace(u'a')
        params = Params({
                u'pretrained_file': unicode(self.FIXTURES_ROOT / u'embeddings/glove.6B.300d.sample.txt.gz'),
                u'embedding_dim': 300,
                u'projection_dim': 20
                })
        embedding_layer = Embedding.from_params(vocab, params)
        input_tensor = torch.LongTensor([[3, 2, 1, 0]])
        embedded = embedding_layer(input_tensor).data.numpy()
        assert embedded.shape == (1, 4, 20)

        input_tensor = torch.LongTensor([[[3, 2, 1, 0]]])
        embedded = embedding_layer(input_tensor).data.numpy()
        assert embedded.shape == (1, 1, 4, 20)

    def test_embedding_layer_actually_initializes_word_vectors_correctly(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace(u"word")
        vocab.add_token_to_namespace(u"word2")
        unicode_space = u"\xa0\xa0\xa0\xa0\xa0\xa0\xa0\xa0"
        vocab.add_token_to_namespace(unicode_space)
        embeddings_filename = unicode(self.TEST_DIR / u"embeddings.gz")
        with gzip.open(embeddings_filename, u'wb') as embeddings_file:
            embeddings_file.write(u"word 1.0 2.3 -1.0\n".encode(u'utf-8'))
            embeddings_file.write("{unicode_space} 3.4 3.3 5.0\n".encode(u'utf-8'))
        params = Params({
                u'pretrained_file': embeddings_filename,
                u'embedding_dim': 3,
                })
        embedding_layer = Embedding.from_params(vocab, params)
        word_vector = embedding_layer.weight.data[vocab.get_token_index(u"word")]
        assert numpy.allclose(word_vector.numpy(), numpy.array([1.0, 2.3, -1.0]))
        word_vector = embedding_layer.weight.data[vocab.get_token_index(unicode_space)]
        assert numpy.allclose(word_vector.numpy(), numpy.array([3.4, 3.3, 5.0]))
        word_vector = embedding_layer.weight.data[vocab.get_token_index(u"word2")]
        assert not numpy.allclose(word_vector.numpy(), numpy.array([1.0, 2.3, -1.0]))

    def test_get_embedding_layer_initializes_unseen_words_randomly_not_zero(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace(u"word")
        vocab.add_token_to_namespace(u"word2")
        embeddings_filename = unicode(self.TEST_DIR / u"embeddings.gz")
        with gzip.open(embeddings_filename, u'wb') as embeddings_file:
            embeddings_file.write(u"word 1.0 2.3 -1.0\n".encode(u'utf-8'))
        params = Params({
                u'pretrained_file': embeddings_filename,
                u'embedding_dim': 3,
                })
        embedding_layer = Embedding.from_params(vocab, params)
        word_vector = embedding_layer.weight.data[vocab.get_token_index(u"word2")]
        assert not numpy.allclose(word_vector.numpy(), numpy.array([0.0, 0.0, 0.0]))

    def test_read_hdf5_format_file(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace(u"word")
        vocab.add_token_to_namespace(u"word2")
        embeddings_filename = unicode(self.TEST_DIR / u"embeddings.hdf5")
        embeddings = numpy.random.rand(vocab.get_vocab_size(), 5)
        with h5py.File(embeddings_filename, u'w') as fout:
            _ = fout.create_dataset(
                    u'embedding', embeddings.shape, dtype=u'float32', data=embeddings
            )

        params = Params({
                u'pretrained_file': embeddings_filename,
                u'embedding_dim': 5,
                })
        embedding_layer = Embedding.from_params(vocab, params)
        assert numpy.allclose(embedding_layer.weight.data.numpy(), embeddings)

    def test_read_hdf5_raises_on_invalid_shape(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace(u"word")
        embeddings_filename = unicode(self.TEST_DIR / u"embeddings.hdf5")
        embeddings = numpy.random.rand(vocab.get_vocab_size(), 10)
        with h5py.File(embeddings_filename, u'w') as fout:
            _ = fout.create_dataset(
                    u'embedding', embeddings.shape, dtype=u'float32', data=embeddings
            )

        params = Params({
                u'pretrained_file': embeddings_filename,
                u'embedding_dim': 5,
                })
        with pytest.raises(ConfigurationError):
            _ = Embedding.from_params(vocab, params)

    def test_read_embedding_file_inside_archive(self):
        token2vec = {
                u"think": torch.Tensor([0.143, 0.189, 0.555, 0.361, 0.472]),
                u"make": torch.Tensor([0.878, 0.651, 0.044, 0.264, 0.872]),
                u"difference": torch.Tensor([0.053, 0.162, 0.671, 0.110, 0.259]),
                u"àèìòù": torch.Tensor([1.0, 2.0, 3.0, 4.0, 5.0])
                }
        vocab = Vocabulary()
        for token in token2vec:
            vocab.add_token_to_namespace(token)

        params = Params({
                u'pretrained_file': unicode(self.FIXTURES_ROOT / u'embeddings/multi-file-archive.zip'),
                u'embedding_dim': 5
                })
        with pytest.raises(ValueError, message=u"No ValueError when pretrained_file is a multi-file archive"):
            Embedding.from_params(vocab, params)

        for ext in [u'.zip', u'.tar.gz']:
            archive_path = unicode(self.FIXTURES_ROOT / u'embeddings/multi-file-archive') + ext
            file_uri = format_embeddings_file_uri(archive_path, u'folder/fake_embeddings.5d.txt')
            params = Params({
                    u'pretrained_file': file_uri,
                    u'embedding_dim': 5
                    })
            embeddings = Embedding.from_params(vocab, params).weight.data
            for tok, vec in list(token2vec.items()):
                i = vocab.get_token_index(tok)
                assert torch.equal(embeddings[i], vec), u'Problem with format ' + archive_path

    def test_embeddings_text_file(self):
        txt_path = unicode(self.FIXTURES_ROOT / u'utf-8_sample/utf-8_sample.txt')

        # This is for sure a correct way to read an utf-8 encoded text file
        with open(txt_path, u'rt', encoding=u'utf-8') as f:
            correct_text = f.read()

        # Check if we get the correct text on plain and compressed versions of the file
        paths = [txt_path] + [txt_path + ext for ext in [u'.gz', u'.zip']]
        for path in paths:
            with EmbeddingsTextFile(path) as f:
                text = f.read()
            assert text == correct_text, u"Test failed for file: " + path

        # Check for a file contained inside an archive with multiple files
        for ext in [u'.zip', u'.tar.gz', u'.tar.bz2', u'.tar.lzma']:
            archive_path = unicode(self.FIXTURES_ROOT / u'utf-8_sample/archives/utf-8') + ext
            file_uri = format_embeddings_file_uri(archive_path, u'folder/utf-8_sample.txt')
            with EmbeddingsTextFile(file_uri) as f:
                text = f.read()
            assert text == correct_text, u"Test failed for file: " + archive_path

        # Passing a second level path when not reading an archive
        with pytest.raises(ValueError):
            with EmbeddingsTextFile(format_embeddings_file_uri(txt_path, u'a/fake/path')):
                pass

    def test_embeddings_text_file_num_tokens(self):
        test_filename = unicode(self.TEST_DIR / u'temp_embeddings.vec')

        def check_num_tokens(first_line, expected_num_tokens):
            with open(test_filename, u'w') as f:
                f.write(first_line)
            with EmbeddingsTextFile(test_filename) as f:
                assert f.num_tokens == expected_num_tokens, "Wrong num tokens for line: {first_line}"

        valid_header_lines = [u'1000000 300', u'300 1000000', u'1000000']
        for line in valid_header_lines:
            check_num_tokens(line, expected_num_tokens=1_000_000)

        not_header_lines = [u'hello 1', u'hello 1 2', u'111 222 333', u'111 222 hello']
        for line in not_header_lines:
            check_num_tokens(line, expected_num_tokens=None)

    def test_decode_embeddings_file_uri(self):
        first_level_paths = [
                u'path/to/embeddings.gz',
                u'unicode/path/òàè+ù.vec',
                u'http://www.embeddings.com/path/to/embeddings.gz',
                u'http://www.embeddings.com/àèìòù?query=blabla.zip',
                ]
        second_level_paths = [
                u'path/to/glove.27B.300d.vec',
                u'òàè+ù.vec',
                u'crawl-300d-2M.vec'
                ]

        for simple_path in first_level_paths:
            assert parse_embeddings_file_uri(simple_path) == (simple_path, None)

        for path1, path2 in izip(first_level_paths, second_level_paths):
            uri = format_embeddings_file_uri(path1, path2)
            decoded = parse_embeddings_file_uri(uri)
            assert decoded == (path1, path2)





from __future__ import division
from __future__ import absolute_import
from allennlp.data.dataset_readers import CcgBankDatasetReader
from allennlp.common.util import ensure_list
from allennlp.common.testing import AllenNlpTestCase


class TestCcgBankReader(AllenNlpTestCase):

    def test_read_from_file(self):

        reader = CcgBankDatasetReader()
        instances = ensure_list(reader.read(self.FIXTURES_ROOT / u'data' / u'ccgbank.txt'))

        assert len(instances) == 2

        instance = instances[0]
        fields = instance.fields
        tokens = [token.text for token in fields[u'tokens'].tokens]
        assert tokens == [u'Pierre', u'Vinken', u',', u'61', u'years', u'old', u',', u'will', u'join', u'the', u'board',
                          u'as', u'a', u'nonexecutive', u'director', u'Nov.', u'29', u'.']

        ccg_categories = fields[u'ccg_categories'].labels
        assert ccg_categories == [u'N/N', u'N', u',', u'N/N', u'N', u'(S[adj]\\NP)\\NP', u',', u'(S[dcl]\\NP)/(S[b]\\NP)',
                                  u'(S[b]\\NP)/NP', u'NP[nb]/N', u'N', u'((S\\NP)\\(S\\NP))/NP', u'NP[nb]/N', u'N/N',
                                  u'N', u'((S\\NP)\\(S\\NP))/N[num]', u'N[num]', u'.']

        original_pos_tags = fields[u'original_pos_tags'].labels
        assert original_pos_tags == [u'NNP', u'NNP', u',', u'CD', u'NNS', u'JJ', u',', u'MD', u'VB', u'DT', u'NN',
                                     u'IN', u'DT', u'JJ', u'NN', u'NNP', u'CD', u'.']

        modified_pos_tags = fields[u'modified_pos_tags'].labels
        assert modified_pos_tags == [u'NNP', u'NNP', u',', u'CD', u'NNS', u'JJ', u',', u'MD', u'VB', u'DT', u'NN',
                                     u'IN', u'DT', u'JJ', u'NN', u'NNP', u'CD', u'.']

        predicate_arg_categories = fields[u'predicate_arg_categories'].labels
        assert predicate_arg_categories == [u'N_73/N_73', u'N', u',', u'N_93/N_93', u'N', u'(S[adj]\\NP_83)\\NP_84',
                                            u',', u'(S[dcl]\\NP_10)/(S[b]_11\\NP_10:B)_11', u'(S[b]\\NP)/NP',
                                            u'NP[nb]_29/N_29', u'N', u'((S_1\\NP_2)_1\\(S_1\\NP_2)_1)/NP',
                                            u'NP[nb]_48/N_48', u'N_43/N_43', u'N',
                                            u'((S_61\\NP_56)_61\\(S_61\\NP_56)_61)/N[num]_62', u'N[num]', u'.']




from __future__ import absolute_import
from allennlp.nn.activations import Activation
from allennlp.nn.initializers import Initializer, InitializerApplicator
from allennlp.nn.regularizers import RegularizerApplicator



# pylint: disable=no-self-use,invalid-name



from __future__ import division
from __future__ import absolute_import
from nltk import Tree

from allennlp.data.dataset_readers.dataset_utils import Ontonotes
from allennlp.common.testing import AllenNlpTestCase


class TestOntonotes(AllenNlpTestCase):

    def test_dataset_iterator(self):
        reader = Ontonotes()
        annotated_sentences = list(reader.dataset_iterator(self.FIXTURES_ROOT / u'conll_2012' / u'subdomain'))
        annotation = annotated_sentences[0]
        assert annotation.document_id == u"test/test/01/test_001"
        assert annotation.sentence_id == 0
        assert annotation.words == [u'Mali', u'government', u'officials', u'say', u'the', u'woman',
                                    u"'s", u'confession', u'was', u'forced', u'.']
        assert annotation.pos_tags == [u'NNP', u'NN', u'NNS', u'VBP', u'DT',
                                       u'NN', u'POS', u'NN', u'VBD', u'JJ', u'.']
        assert annotation.word_senses == [None, None, 1, 1, None, 2, None, None, 1, None, None]
        assert annotation.predicate_framenet_ids == [None, None, None, u'01', None,
                                                     None, None, None, u'01', None, None]
        assert annotation.srl_frames == [(u"say", [u'B-ARG0', u'I-ARG0', u'I-ARG0', u'B-V', u'B-ARG1',
                                                  u'I-ARG1', u'I-ARG1', u'I-ARG1', u'I-ARG1', u'I-ARG1', u'O']),
                                         (u"was", [u'O', u'O', u'O', u'O', u'B-ARG1', u'I-ARG1', u'I-ARG1',
                                                  u'I-ARG1', u'B-V', u'B-ARG2', u'O'])]
        assert annotation.named_entities == [u'B-GPE', u'O', u'O', u'O', u'O', u'O',
                                             u'O', u'O', u'O', u'O', u'O']
        assert annotation.predicate_lemmas == [None, None, u'official', u'say', None,
                                               u'man', None, None, u'be', None, None]
        assert annotation.speakers == [None, None, None, None, None, None,
                                       None, None, None, None, None]

        assert annotation.parse_tree == Tree.fromstring(u"(TOP(S(NP(NML (NNP Mali)  (NN government) )"
                                                        u" (NNS officials) )(VP (VBP say) (SBAR(S(NP(NP"
                                                        u" (DT the)  (NN woman)  (POS 's) ) (NN "
                                                        u"confession) )(VP (VBD was) (ADJP (JJ "
                                                        u"forced) ))))) (. .) ))")
        assert annotation.coref_spans == set([(1, (4, 6)), (3, (4, 7))])

        annotation = annotated_sentences[1]
        assert annotation.document_id == u"test/test/02/test_002"
        assert annotation.sentence_id == 0
        assert annotation.words == [u'The', u'prosecution', u'rested', u'its', u'case', u'last', u'month',
                                    u'after', u'four', u'months', u'of', u'hearings', u'.']
        assert annotation.pos_tags == [u'DT', u'NN', u'VBD', u'PRP$', u'NN', u'JJ', u'NN',
                                       u'IN', u'CD', u'NNS', u'IN', u'NNS', u'.']
        assert annotation.word_senses == [None, 2, 5, None, 2, None, None,
                                          None, None, 1, None, 1, None]
        assert annotation.predicate_framenet_ids == [None, None, u'01', None, None, None,
                                                     None, None, None, None, None, u'01', None]
        assert annotation.srl_frames == [(u'rested', [u'B-ARG0', u'I-ARG0', u'B-V', u'B-ARG1',
                                                     u'I-ARG1', u'B-ARGM-TMP', u'I-ARGM-TMP',
                                                     u'B-ARGM-TMP', u'I-ARGM-TMP', u'I-ARGM-TMP',
                                                     u'I-ARGM-TMP', u'I-ARGM-TMP', u'O']),
                                         (u'hearings', [u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O',
                                                       u'O', u'O', u'O', u'B-V', u'O'])]
        assert annotation.named_entities == [u'O', u'O', u'O', u'O', u'O', u'B-DATE', u'I-DATE',
                                             u'O', u'B-DATE', u'I-DATE', u'O', u'O', u'O']
        assert annotation.predicate_lemmas == [None, u'prosecution', u'rest', None, u'case',
                                               None, None, None, None, u'month', None, u'hearing', None]
        assert annotation.speakers == [None, None, None, None, None, None,
                                       None, None, None, None, None, None, None]
        assert annotation.parse_tree == Tree.fromstring(u"(TOP(S(NP (DT The)  (NN prosecution) )(VP "
                                                        u"(VBD rested) (NP (PRP$ its)  (NN case) )"
                                                        u"(NP (JJ last)  (NN month) )(PP (IN after) "
                                                        u"(NP(NP (CD four)  (NNS months) )(PP (IN"
                                                        u" of) (NP (NNS hearings) ))))) (. .) ))")
        assert annotation.coref_spans == set([(2, (0, 1)), (2, (3, 3))])

        # Check we can handle sentences without verbs.
        annotation = annotated_sentences[2]
        assert annotation.document_id == u'test/test/03/test_003'
        assert annotation.sentence_id == 0
        assert annotation.words == [u'Denise', u'Dillon', u'Headline', u'News', u'.']
        assert annotation.pos_tags == [u'NNP', u'NNP', u'NNP', u'NNP', u'.']
        assert annotation.word_senses == [None, None, None, None, None]
        assert annotation.predicate_framenet_ids == [None, None, None, None, None]
        assert annotation.srl_frames == []
        assert annotation.named_entities == [u'B-PERSON', u'I-PERSON',
                                             u'B-WORK_OF_ART', u'I-WORK_OF_ART', u'O']
        assert annotation.predicate_lemmas == [None, None, None, None, None]
        assert annotation.speakers == [None, None, None, None, None]
        assert annotation.parse_tree == Tree.fromstring(u"(TOP(FRAG(NP (NNP Denise) "
                                                        u" (NNP Dillon) )(NP (NNP Headline)  "
                                                        u"(NNP News) ) (. .) ))")
        assert annotation.coref_spans == set([(2, (0, 1))])

        # Check we can handle sentences with 2 identical verbs.
        annotation = annotated_sentences[3]
        assert annotation.document_id == u'test/test/04/test_004'
        assert annotation.sentence_id == 0
        assert annotation.words == [u'and', u'that', u'wildness', u'is', u'still', u'in', u'him', u',',
                                    u'as', u'it', u'is', u'with', u'all', u'children', u'.']
        assert annotation.pos_tags == [u'CC', u'DT', u'NN', u'VBZ', u'RB', u'IN', u'PRP', u',',
                                       u'IN', u'PRP', u'VBZ', u'IN', u'DT', u'NNS', u'.']
        assert annotation.word_senses == [None, None, None, 4.0, None, None, None, None,
                                          None, None, 5.0, None, None, None, None]
        assert annotation.predicate_framenet_ids == [None, None, None, u'01', None, None,
                                                     None, None, None, None, u'01', None, None, None, None]
        assert annotation.srl_frames == [(u'is', [u'B-ARGM-DIS', u'B-ARG1', u'I-ARG1',
                                                 u'B-V', u'B-ARGM-TMP', u'B-ARG2', u'I-ARG2',
                                                 u'O', u'B-ARGM-ADV', u'I-ARGM-ADV', u'I-ARGM-ADV',
                                                 u'I-ARGM-ADV', u'I-ARGM-ADV', u'I-ARGM-ADV', u'O']),
                                         (u'is', [u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O',
                                                 u'B-ARG1', u'B-V', u'B-ARG2', u'I-ARG2', u'I-ARG2', u'O'])]
        assert annotation.named_entities == [u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O',
                                             u'O', u'O', u'O', u'O', u'O', u'O', u'O']
        assert annotation.predicate_lemmas == [None, None, None, u'be', None, None, None,
                                               None, None, None, u'be', None, None, None, None]
        assert annotation.speakers == [u'_Avalon_', u'_Avalon_', u'_Avalon_', u'_Avalon_', u'_Avalon_',
                                       u'_Avalon_', u'_Avalon_', u'_Avalon_', u'_Avalon_', u'_Avalon_',
                                       u'_Avalon_', u'_Avalon_', u'_Avalon_', u'_Avalon_', u'_Avalon_']
        assert annotation.parse_tree == Tree.fromstring(u"(TOP (S (CC and) (NP (DT that) (NN wildness)) "
                                                        u"(VP (VBZ is) (ADVP (RB still)) (PP (IN in) (NP "
                                                        u"(PRP him))) (, ,) (SBAR (IN as) (S (NP (PRP it)) "
                                                        u"(VP (VBZ is) (PP (IN with) (NP (DT all) (NNS "
                                                        u"children))))))) (. .)))")
        assert annotation.coref_spans == set([(14, (6, 6))])

    def test_dataset_path_iterator(self):
        reader = Ontonotes()
        files = list(reader.dataset_path_iterator(self.FIXTURES_ROOT / u'conll_2012'))
        expected_paths = [unicode(self.FIXTURES_ROOT / u'conll_2012' / u'subdomain' / u'example.gold_conll'),
                          unicode(self.FIXTURES_ROOT / u'conll_2012' / u'subdomain2' / u'example.gold_conll')]
        assert len(files) == len(expected_paths)
        assert set(files) == set(expected_paths)

    def test_ontonotes_can_read_conll_file_with_multiple_documents(self):
        reader = Ontonotes()
        file_path = self.FIXTURES_ROOT / u'coref' / u'coref.gold_conll'
        documents = list(reader.dataset_document_iterator(file_path))
        assert len(documents) == 2




from __future__ import absolute_import
#typing

#overrides
import torch
from torch.nn import Dropout

from allennlp.modules.feedforward import FeedForward
from allennlp.modules.layer_norm import LayerNorm
from allennlp.modules.seq2seq_encoders.multi_head_self_attention import MultiHeadSelfAttention
from allennlp.modules.seq2seq_encoders.seq2seq_encoder import Seq2SeqEncoder
from allennlp.nn.activations import Activation
from allennlp.nn.util import add_positional_features
try:
    from itertools import izip
except:
    izip = zip



class StackedSelfAttentionEncoder(Seq2SeqEncoder):
    # pylint: disable=line-too-long
    u"""
    Implements a stacked self-attention encoder similar to the Transformer
    architecture in `Attention is all you Need
    <https://www.semanticscholar.org/paper/Attention-Is-All-You-Need-Vaswani-Shazeer/0737da0767d77606169cbf4187b83e1ab62f6077>`_ .

    This encoder combines 3 layers in a 'block':

    1. A 2 layer FeedForward network.
    2. Multi-headed self attention, which uses 2 learnt linear projections
       to perform a dot-product similarity between every pair of elements
       scaled by the square root of the sequence length.
    3. Layer Normalisation.

    These are then stacked into ``num_layers`` layers.

    Parameters
    ----------
    input_dim : ``int``, required.
        The input dimension of the encoder.
    hidden_dim : ``int``, required.
        The hidden dimension used for the _input_ to self attention layers
        and the _output_ from the feedforward layers.
    projection_dim : ``int``, required.
        The dimension of the linear projections for the self-attention layers.
    feedforward_hidden_dim : ``int``, required.
        The middle dimension of the FeedForward network. The input and output
        dimensions are fixed to ensure sizes match up for the self attention layers.
    num_layers : ``int``, required.
        The number of stacked self attention -> feedfoward -> layer normalisation blocks.
    num_attention_heads : ``int``, required.
        The number of attention heads to use per layer.
    use_positional_encoding: ``bool``, optional, (default = True)
        Whether to add sinusoidal frequencies to the input tensor. This is strongly recommended,
        as without this feature, the self attention layers have no idea of absolute or relative
        position (as they are just computing pairwise similarity between vectors of elements),
        which can be important features for many tasks.
    dropout_prob : ``float``, optional, (default = 0.1)
        The dropout probability for the feedforward network.
    residual_dropout_prob : ``float``, optional, (default = 0.2)
        The dropout probability for the residual connections.
    attention_dropout_prob : ``float``, optional, (default = 0.1)
        The dropout probability for the attention distributions in each attention layer.
    """
    def __init__(self,
                 input_dim     ,
                 hidden_dim     ,
                 projection_dim     ,
                 feedforward_hidden_dim     ,
                 num_layers     ,
                 num_attention_heads     ,
                 use_positional_encoding       = True,
                 dropout_prob        = 0.1,
                 residual_dropout_prob        = 0.2,
                 attention_dropout_prob        = 0.1)        :
        super(StackedSelfAttentionEncoder, self).__init__()

        self._use_positional_encoding = use_positional_encoding
        self._attention_layers = []
        self._feedfoward_layers = []
        self._layer_norm_layers = []
        self._feed_forward_layer_norm_layers = []

        feedfoward_input_dim = input_dim
        for i in range(num_layers):
            feedfoward = FeedForward(feedfoward_input_dim,
                                     activations=[Activation.by_name(u'relu')(),
                                                  Activation.by_name(u'linear')()],
                                     hidden_dims=[feedforward_hidden_dim, hidden_dim],
                                     num_layers=2,
                                     dropout=dropout_prob)

            self.add_module("feedforward_{i}", feedfoward)
            self._feedfoward_layers.append(feedfoward)

            feedforward_layer_norm = LayerNorm(feedfoward.get_output_dim())
            self.add_module("feedforward_layer_norm_{i}", feedforward_layer_norm)
            self._feed_forward_layer_norm_layers.append(feedforward_layer_norm)

            self_attention = MultiHeadSelfAttention(num_heads=num_attention_heads,
                                                    input_dim=hidden_dim,
                                                    attention_dim=projection_dim,
                                                    values_dim=projection_dim,
                                                    attention_dropout_prob=attention_dropout_prob)
            self.add_module("self_attention_{i}", self_attention)
            self._attention_layers.append(self_attention)

            layer_norm = LayerNorm(self_attention.get_output_dim())
            self.add_module("layer_norm_{i}", layer_norm)
            self._layer_norm_layers.append(layer_norm)

            feedfoward_input_dim = hidden_dim

        self.dropout = Dropout(residual_dropout_prob)
        self._input_dim = input_dim
        self._output_dim = self._attention_layers[-1].get_output_dim()

    #overrides
    def get_input_dim(self)       :
        return self._input_dim

    #overrides
    def get_output_dim(self)       :
        return self._output_dim

    #overrides
    def is_bidirectional(self):
        return False

    #overrides
    def forward(self, inputs              , mask              ): # pylint: disable=arguments-differ
        if self._use_positional_encoding:
            output = add_positional_features(inputs)
        else:
            output = inputs
        for (attention,
             feedforward,
             feedforward_layer_norm,
             layer_norm) in izip(self._attention_layers,
                                self._feedfoward_layers,
                                self._feed_forward_layer_norm_layers,
                                self._layer_norm_layers):
            cached_input = output
            # Project output of attention encoder through a feedforward
            # network and back to the input size for the next layer.
            # shape (batch_size, timesteps, input_size)
            feedforward_output = feedforward(output)
            feedforward_output = self.dropout(feedforward_output)
            if feedforward_output.size() == cached_input.size():
                # First layer might have the wrong size for highway
                # layers, so we exclude it here.
                feedforward_output = feedforward_layer_norm(feedforward_output + cached_input)
            # shape (batch_size, sequence_length, hidden_dim)
            attention_output = attention(feedforward_output, mask)
            output = layer_norm(self.dropout(attention_output) + feedforward_output)

        return output

StackedSelfAttentionEncoder = Seq2SeqEncoder.register(u"stacked_self_attention")(StackedSelfAttentionEncoder)



# python
#
# See the accompanying LICENSE file.
#
# Find things that haven't been documented and should be or have been
# but don't exist.

import glob, sys

import apsw

retval=0

classes={}

for filename in glob.glob("doc/*.rst"):
    for line in open(filename, "rtU"):
        line=line.strip().split()

        if len(line)>=2:
            if line[0]==".." and line[1] in ("method::", "automethod::", "attribute::"):
                funcname=line[2].split("(")[0].strip()

                if "." in funcname:
                    klass, funcname=funcname.split(".",1)
                else:
                    klass="apsw"
                if klass not in classes:
                    classes[klass]=[]
                classes[klass].append(funcname)


# ok, so we know what was documented.  Now lets see what exists

con=apsw.Connection(":memory:")
cur=con.cursor()
cur.execute("create table x(y); insert into x values(x'abcdef1012');select * from x")
blob=con.blobopen("main", "x", "y", con.last_insert_rowid(), 0)
vfs=apsw.VFS("aname", "")
vfsfile=apsw.VFSFile("", ":memory:", [apsw.SQLITE_OPEN_MAIN_DB|apsw.SQLITE_OPEN_CREATE|apsw.SQLITE_OPEN_READWRITE, 0])

# virtual tables aren't real - just check their size hasn't changed
assert len(classes['VTModule'])==2
del classes['VTModule']
assert len(classes['VTTable'])==13
del classes['VTTable']
assert len(classes['VTCursor'])==6
del classes['VTCursor']

for name, obj in ( ('Connection', con),
                   ('Cursor', cur),
                   ('blob', blob),
                   ('VFS', vfs),
                   ('VFSFile', vfsfile),
                   ('apsw', apsw),
                   ):
    if name not in classes:
        retval=1
        print "class", name,"not found"
        continue

    for c in classes[name]:
        if not hasattr(obj, c):
            # it is legit for these to be missing from code (currently because code is broken)
            if (name+"."+c) in ("apsw.async_control", "apsw.async_initialize", "apsw.async_run", "apsw.async_shutdown"):
                continue
            retval=1
            print "%s.%s in documentation but not object" % (name, c)
    for c in dir(obj):
        if c.startswith("__"): continue
        if name=="apsw":
            # ignore constants and modules
            if type(getattr(apsw, c)) in (type(3), type(sys)):
                continue
            # ignore debugging thingies
            if c.startswith("test_") or c in ("faultdict", "_fini"):
                continue
            # ignore the exceptions
            if isinstance(getattr(apsw, c), type) and issubclass(getattr(apsw,c), Exception):
                continue
            # ignore classes !!!
            if c in ("Connection", "VFS", "VFSFile", "zeroblob", "Shell", "URIFilename"):
                continue
            # ignore mappings !!!
            if c.startswith("mapping_"):
                continue
        if c not in classes[name]:
            if "%s.%s" % (name, c)  not in ("Cursor.next",):
                retval=1
                print "%s.%s on object but not in documentation" % (name, c)

sys.exit(retval)
